@@ -16,14 +16,13 @@ from werkzeug.exceptions import BadRequest
 
 from zish import dumps
 
-import chellow.bank_holidays
-import chellow.utils
 from chellow.models import (
     Bill,
     BillType,
     Channel,
     ClockInterval,
     Contract,
+    DtcMeterType,
     Era,
     HhDatum,
     Llfc,
@@ -63,8 +62,8 @@ cons_types = ["construction", "commissioning", "operation"]
 lec_cats = list(
     (f"{v}-kwh", f"hist-{v}-kwh")
     for v in [
-        "import-net",
-        "export-net",
+        "import-grid",
+        "export-grid",
         "import-gen",
         "export-gen",
         "import-3rd-party",
@@ -121,18 +120,40 @@ def contract_func(caches, contract, func_name):
     return ns.get(func_name, None)
 
 
-def non_core_rate(sess, caches, contract_id_or_name, date):
-    return hh_rate(sess, caches, contract_id_or_name, date, market_role_code="Z")
+def non_core_rate(sess, caches, contract_id_or_name, date, exact=False):
+    return hh_rate(
+        sess, caches, contract_id_or_name, date, market_role_code="Z", exact=exact
+    )
 
 
-def hh_rate(sess, caches, contract_id_or_name, date, market_role_code=None):
+def hh_rate(
+    sess, caches, contract_id_or_name, date, market_role_code=None, exact=False
+):
     try:
-        return caches["computer"]["rates"][market_role_code][contract_id_or_name][date]
+        ccache = caches["computer"]
+    except KeyError:
+        ccache = caches["computer"] = {}
+
+    try:
+        contract_id = ccache[market_role_code][contract_id_or_name]
     except KeyError:
         try:
-            ccache = caches["computer"]
+            mr_cache = ccache[market_role_code]
         except KeyError:
-            ccache = caches["computer"] = {}
+            mr_cache = ccache[market_role_code] = {}
+
+        if market_role_code is None:
+            contract_id = Contract.get_by_id(sess, contract_id_or_name).id
+        else:
+            contract_id = Contract.get_by_role_code_name(
+                sess, market_role_code, contract_id_or_name
+            ).id
+
+        mr_cache[contract_id_or_name] = contract_id
+
+    try:
+        return ccache["rates"][contract_id][date]
+    except KeyError:
 
         try:
             rss_cache = ccache["rates"]
@@ -140,24 +161,14 @@ def hh_rate(sess, caches, contract_id_or_name, date, market_role_code=None):
             rss_cache = ccache["rates"] = {}
 
         try:
-            mr_cache = rss_cache[market_role_code]
+            cont_cache = rss_cache[contract_id]
         except KeyError:
-            mr_cache = rss_cache[market_role_code] = {}
-
-        try:
-            cont_cache = mr_cache[contract_id_or_name]
-        except KeyError:
-            cont_cache = mr_cache[contract_id_or_name] = {}
+            cont_cache = rss_cache[contract_id] = {}
 
         try:
             return cont_cache[date]
         except KeyError:
-            if market_role_code is None:
-                contract = Contract.get_by_id(sess, contract_id_or_name)
-            else:
-                contract = Contract.get_by_role_code_name(
-                    sess, market_role_code, contract_id_or_name
-                )
+            contract = Contract.get_by_id(sess, contract_id)
 
             year_after = date + YEAR
             year_before = date - YEAR
@@ -175,18 +186,20 @@ def hh_rate(sess, caches, contract_id_or_name, date, market_role_code=None):
             )
 
             if rs is None:
-                rs = (
+                last_rs = (
                     sess.query(RateScript)
                     .filter(RateScript.contract == contract)
                     .order_by(RateScript.start_date.desc())
                     .first()
                 )
-                if date < rs.start_date:
+                if date < last_rs.start_date:
                     cstart = year_before
-                    cfinish = min(year_after, rs.start_date - HH)
+                    cfinish = min(year_after, last_rs.start_date - HH)
                 else:
-                    cstart = max(rs.finish_date + HH, year_before)
+                    cstart = max(last_rs.finish_date + HH, year_before)
                     cfinish = year_after
+                if not exact:
+                    rs = last_rs
             else:
                 cstart = max(rs.start_date, year_before)
                 if rs.finish_date is None:
@@ -194,27 +207,26 @@ def hh_rate(sess, caches, contract_id_or_name, date, market_role_code=None):
                 else:
                     cfinish = min(rs.finish_date, year_after)
 
-            market_role_code = rs.contract.market_role.code
-            if market_role_code == "M":
-                seg = "/e/mop_rate_scripts/"
-            elif market_role_code == "C":
-                seg = "/e/dc_rate_scripts/"
-            elif market_role_code == "X":
-                seg = "/e/supplier_rate_scripts/"
-            elif market_role_code == "Z":
-                seg = "/non_core_rate_scripts/"
-            elif market_role_code == "R":
-                seg = "/e/dno_rate_scripts/"
+            if rs is None:
+                vals = None
             else:
-                raise Exception(
-                    f"The market role code {market_role_code} isn't recognized."
-                )
+                market_role_code = rs.contract.market_role.code
+                if market_role_code == "M":
+                    seg = "/e/mop_rate_scripts/"
+                elif market_role_code == "C":
+                    seg = "/e/dc_rate_scripts/"
+                elif market_role_code == "X":
+                    seg = "/e/supplier_rate_scripts/"
+                elif market_role_code == "Z":
+                    seg = "/non_core_rate_scripts/"
+                elif market_role_code == "R":
+                    seg = "/e/dno_rate_scripts/"
+                else:
+                    raise Exception(
+                        f"The market role code {market_role_code} isn't recognized."
+                    )
 
-            vals = PropDict(
-                f"the rate script {chellow.utils.url_root}{seg}{rs.id} ",
-                loads(rs.script),
-                [],
-            )
+                vals = PropDict(f"the rate script {seg}{rs.id} ", loads(rs.script), [])
             for dt in hh_range(caches, cstart, cfinish):
                 if dt not in cont_cache:
                     cont_cache[dt] = vals
@@ -265,9 +277,9 @@ def displaced_era(
         era = site_era.era
         source_code = era.supply.source.code
         if site_era.is_physical and (
-            source_code in ("gen", "gen-net")
+            source_code in ("gen", "gen-grid")
             or (
-                source_code == "net"
+                source_code == "grid"
                 and sess.query(Channel)
                 .filter(
                     Channel.era == era,
@@ -280,7 +292,7 @@ def displaced_era(
         ):
             has_displaced = True
 
-        if source_code in ("net", "gen-net") and era.imp_mpan_core is not None:
+        if source_code in ("grid", "gen-grid") and era.imp_mpan_core is not None:
             eras[
                 "_".join(
                     (
@@ -407,6 +419,17 @@ def cache_level(cache, key):
         return new_cache
 
 
+def cached_dno_contract(sess, caches, dno_code):
+    try:
+        return caches["computer"]["dno_contracts"][dno_code]
+    except KeyError:
+        computer_cache = cache_level(caches, "computer")
+        cont_cache = cache_level(computer_cache, "dno_contracts")
+        dno_contract = Contract.get_dno_by_name(sess, dno_code)
+        cont_cache[dno_code] = dno_contract
+        return dno_contract
+
+
 def datum_range(sess, caches, years_back, start_date, finish_date):
     try:
         return caches["computer"]["datum_range"][years_back][start_date][finish_date]
@@ -471,8 +494,8 @@ def datum_range(sess, caches, years_back, start_date, finish_date):
                         "exp-msp-kvar": 0,
                         "msp-kw": 0,
                         "msp-kwh": 0,
-                        "hist-import-net-kvarh": 0,
-                        "hist-export-net-kvarh": 0,
+                        "hist-import-grid-kvarh": 0,
+                        "hist-export-grid-kvarh": 0,
                         "anti-msp-kwh": 0,
                         "anti-msp-kw": 0,
                         "imp-msp-kvarh": 0,
@@ -541,6 +564,7 @@ class DataSource:
         self.era_map_mop_contracts = self.era_map.get("mop_contracts", {})
         self.era_map_cops = self.era_map.get("cops", {})
         self.era_map_comms = self.era_map.get("comms", {})
+        self.era_map_dtc_meter_types = self.era_map.get("dtc_meter_types", {})
         self.era_map_mpan_cores = self.era_map.get("mpan_cores", {})
         self.era_map_dnos = self.era_map.get("dnos", {})
         self.era_map_gsp_groups = self.era_map.get("gsp_groups", {})
@@ -558,9 +582,9 @@ class DataSource:
             self.rate_cache = self.caches["computer"]["rates"]
             return val
 
-    def non_core_rate(self, contract_name_or_id, date):
-        return hh_rate(
-            self.sess, self.caches, contract_name_or_id, date, market_role_code="Z"
+    def non_core_rate(self, contract_name_or_id, date, exact=False):
+        return non_core_rate(
+            self.sess, self.caches, contract_name_or_id, date, exact=exact
         )
 
     def _add_problem(self, problem):
@@ -606,7 +630,7 @@ class SiteSource(DataSource):
             self.mpan_core = era.imp_mpan_core
             self.dno = self.supply.dno
             self.dno_code = self.dno.dno_code
-            self.dno_contract = Contract.get_dno_by_name(self.sess, self.dno_code)
+            self.dno_contract = cached_dno_contract(self.sess, caches, self.dno_code)
 
             era_map_llfcs = self.era_map_llfcs.get(self.dno_code, {})
             if era.imp_llfc.code in era_map_llfcs:
@@ -663,9 +687,23 @@ class SiteSource(DataSource):
             else:
                 self.sc = era.imp_sc
 
-            self.properties = self.era_map.get("properties_overwritten", {})
-            self.properties.update(era.props)
-            self.properties.update(self.era_map.get("properties_overwrite", {}))
+            if era.dtc_meter_type is None:
+                era_dtc_meter_type_code = None
+            else:
+                era_dtc_meter_type_code = era.dtc_meter_type.code
+            if era_dtc_meter_type_code in self.era_map_dtc_meter_types:
+                self.dtc_meter_type_code = self.era_map_dtc_meter_types[
+                    era_dtc_meter_type_code
+                ]
+                if self.dtc_meter_type_code is None:
+                    self.dtc_meter_type = None
+                else:
+                    self.dtc_meter_type = DtcMeterType.get_by_code(
+                        sess, era_dtc_meter_type_code
+                    )
+            else:
+                self.dtc_meter_type = era.dtc_meter_type
+                self.dtc_meter_type_code = era_dtc_meter_type_code
 
             self.is_import = True
             self.voltage_level_code = self.llfc.voltage_level.code
@@ -673,6 +711,7 @@ class SiteSource(DataSource):
             self.gsp_group_code = self.supply.gsp_group.code
             self.ssc = self.era.ssc
             self.ssc_code = None if self.ssc is None else self.ssc.code
+            self.energisation_status_code = era.energisation_status.code
 
         era_q = (
             select(Era.id)
@@ -722,8 +761,8 @@ class SiteSource(DataSource):
         hist_map = {}
 
         for hist_date in hh_range(self.caches, self.history_start, self.history_finish):
-            export_net_kwh = 0
-            import_net_kwh = 0
+            export_grid_kwh = 0
+            import_grid_kwh = 0
             export_gen_kwh = 0
             import_gen_kwh = 0
             import_3rd_party_kwh = 0
@@ -731,16 +770,16 @@ class SiteSource(DataSource):
             export_3rd_party_kwh = 0
             while hh_start_date == hist_date:
                 statuses.add(status)
-                if not imp_related and source_code in ("net", "gen-net"):
-                    export_net_kwh += hh_value
-                if imp_related and source_code in ("net", "gen-net"):
-                    import_net_kwh += hh_value
+                if not imp_related and source_code in ("grid", "gen-grid"):
+                    export_grid_kwh += hh_value
+                if imp_related and source_code in ("grid", "gen-grid"):
+                    import_grid_kwh += hh_value
                 if (imp_related and source_code == "gen") or (
-                    not imp_related and source_code == "gen-net"
+                    not imp_related and source_code == "gen-grid"
                 ):
                     import_gen_kwh += hh_value
                 if (not imp_related and source_code == "gen") or (
-                    imp_related and source_code == "gen-net"
+                    imp_related and source_code == "gen-grid"
                 ):
                     export_gen_kwh += hh_value
                 if (imp_related and source_code == "3rd-party") or (
@@ -763,8 +802,8 @@ class SiteSource(DataSource):
 
             hh_values = {
                 "status": status,
-                "hist-import-net-kwh": import_net_kwh,
-                "hist-export-net-kwh": export_net_kwh,
+                "hist-import-grid-kwh": import_grid_kwh,
+                "hist-export-grid-kwh": export_grid_kwh,
                 "hist-import-gen-kwh": import_gen_kwh,
                 "hist-export-gen-kwh": export_gen_kwh,
                 "hist-import-3rd-party-kwh": import_3rd_party_kwh,
@@ -776,7 +815,7 @@ class SiteSource(DataSource):
             hh_values["hist-kwh"] = hh_values["hist-used-gen-msp-kwh"] = (
                 hh_values["hist-import-gen-kwh"]
                 - hh_values["hist-export-gen-kwh"]
-                - hh_values["hist-export-net-kwh"]
+                - hh_values["hist-export-grid-kwh"]
             )
 
             hh_values["msp-kwh"] = hh_values["used-gen-msp-kwh"] = hh_values[
@@ -787,12 +826,12 @@ class SiteSource(DataSource):
 
             hh_values["hist-used-kwh"] = (
                 hh_values["hist-used-gen-msp-kwh"]
-                + hh_values["hist-import-net-kwh"]
+                + hh_values["hist-import-grid-kwh"]
                 + hh_values["hist-used-3rd-party-kwh"]
             )
 
             hh_values["used-kwh"] = hh_values["hist-used-kwh"]
-            hh_values["import-net-kwh"] = hh_values["hist-import-net-kwh"]
+            hh_values["import-grid-kwh"] = hh_values["hist-import-grid-kwh"]
             hh_values["msp-kw"] = hh_values["used-gen-msp-kw"] = (
                 hh_values["used-gen-msp-kwh"] * 2
             )
@@ -804,6 +843,10 @@ class SiteSource(DataSource):
         ):
             datum = dtm.copy()
             datum.update(hist_map[datum["hist-start"]])
+            datum["msp-kva"] = (
+                datum["msp-kw"] ** 2
+                + max(datum["imp-msp-kvar"], datum["exp-msp-kvar"]) ** 2
+            ) ** 0.5
             self.hh_data.append(datum)
             self.supplier_bill_hhs[dtm["start-date"]] = {}
             self.mop_bill_hhs[dtm["start-date"]] = {}
@@ -813,8 +856,8 @@ class SiteSource(DataSource):
             for hh in self.hh_data:
                 try:
                     delt_hh = self.deltas["hhs"][hh["start-date"]]
-                    hh["import-net-kwh"] = delt_hh["import-net-kwh"]
-                    hh["export-net-kwh"] = delt_hh["export-net-kwh"]
+                    hh["import-grid-kwh"] = delt_hh["import-grid-kwh"]
+                    hh["export-grid-kwh"] = delt_hh["export-grid-kwh"]
                     hh["import-gen-kwh"] = delt_hh["import-gen-kwh"]
                     hh["msp-kwh"] = delt_hh["msp-kwh"]
                     hh["used-kwh"] = delt_hh["used-kwh"]
@@ -924,7 +967,7 @@ class SupplySource(DataSource):
 
         self.is_displaced = False
         self.bill = bill
-        if self.bill is not None:
+        if self.bill is not None and hasattr(self.bill, "start_date"):
             self.bill_start = bill.start_date
             self.bill_finish = bill.finish_date
             self.is_last_bill_gen = (
@@ -955,7 +998,7 @@ class SupplySource(DataSource):
             self.dno = self.supply.dno
             self.dno_code = self.dno.dno_code
 
-        self.dno_contract = Contract.get_dno_by_name(sess, self.dno_code)
+        self.dno_contract = cached_dno_contract(self.sess, caches, self.dno_code)
 
         era_map_llfcs = self.era_map_llfcs.get(self.dno_code, {})
         self.is_import = is_import
@@ -971,7 +1014,10 @@ class SupplySource(DataSource):
             else:
                 self.llfc = era.imp_llfc
 
-            self.sc = era.imp_sc
+            if "imp_sc" in self.era_map:
+                self.sc = self.era_map["imp_sc"]
+            else:
+                self.sc = era.imp_sc
             self.supplier_account = era.imp_supplier_account
 
             if era.imp_supplier_contract.id in self.era_map_supplier_contracts:
@@ -998,7 +1044,10 @@ class SupplySource(DataSource):
             else:
                 self.llfc = era.exp_llfc
 
-            self.sc = 0 if era.exp_sc is None else era.exp_sc
+            if "exp_sc" in self.era_map:
+                self.sc = self.era_map["exp_sc"]
+            else:
+                self.sc = era.exp_sc
             self.supplier_account = era.exp_supplier_account
 
             if era.exp_supplier_contract is None:
@@ -1037,9 +1086,23 @@ class SupplySource(DataSource):
         else:
             self.comm_code = era.comm.code
 
-        self.properties = dict(self.era_map.get("properties_overwritten", {}))
-        self.properties.update(era.props)
-        self.properties.update(self.era_map.get("properties_overwrite", {}))
+        if era.dtc_meter_type is None:
+            era_dtc_meter_type_code = None
+        else:
+            era_dtc_meter_type_code = era.dtc_meter_type.code
+        if era_dtc_meter_type_code in self.era_map_dtc_meter_types:
+            self.dtc_meter_type_code = self.era_map_dtc_meter_types[
+                era_dtc_meter_type_code
+            ]
+            if self.dtc_meter_type_code is None:
+                self.dtc_meter_type_code = None
+            else:
+                self.dtc_meter_type = DtcMeterType.get_by_code(
+                    sess, self.dtc_meter_type_code
+                )
+        else:
+            self.dtc_meter_type = era.dtc_meter_type
+            self.dtc_meter_type_code = era_dtc_meter_type_code
 
         self.id = self.mpan_core
         self.msn = era.msn
@@ -1148,11 +1211,15 @@ class SupplySource(DataSource):
                         - Datetime(chunk_start.year, 1, 1)
                     ).total_seconds()
                 )
+                if hist_era.pc.code == "00":
+                    ssc = Ssc.get_by_code(sess, "0393", chunk_start)
+                else:
+                    ssc = hist_era.ssc
 
-                for tpr in (
-                    sess.query(Tpr)
+                for tpr in sess.scalars(
+                    select(Tpr)
                     .join(MeasurementRequirement)
-                    .filter(MeasurementRequirement.ssc == hist_era.ssc)
+                    .where(MeasurementRequirement.ssc == ssc)
                 ):
                     for hist_date in hh_range(self.caches, chunk_start, chunk_finish):
                         if is_tpr(sess, self.caches, tpr.code, hist_date):
@@ -1349,6 +1416,10 @@ class SupplySource(DataSource):
         ):
             datum = d.copy()
             datum.update(hist_map.get(d["hist-start"], {}))
+            datum["msp-kva"] = (
+                datum["msp-kw"] ** 2
+                + max(datum["imp-msp-kvar"], datum["exp-msp-kvar"]) ** 2
+            ) ** 0.5
             self.hh_data.append(datum)
 
             d_start = d["start-date"]
@@ -1523,6 +1594,16 @@ def _find_hhs(caches, sess, pairs, chunk_start, chunk_finish):
     if hh_after(pairs[-1]["start-date"], chunk_finish):
         del pairs[-1]
 
+    # If nothing left
+    if len(pairs) == 0:
+        pairs.append(
+            {
+                "start-date": chunk_start,
+                "finish-date": chunk_finish,
+                "tprs": {"00001": 0},
+            }
+        )
+
     # squash
     if hh_before(pairs[0]["start-date"], chunk_start):
         pairs[0]["start-date"] = chunk_start
@@ -1577,7 +1658,7 @@ def _read_generator(sess, supply, start, is_forwards, is_prev):
             Bill.supply == supply,
             BillType.code != "W",
         )
-        .options(joinedload(RegisterRead.bill))
+        .options(joinedload(RegisterRead.bill), joinedload(r_typ))
     )
 
     if is_forwards:
@@ -1614,18 +1695,6 @@ def _read_generator(sess, supply, start, is_forwards, is_prev):
         if bill.id != r.bill.id:
             continue
 
-        era = supply.find_era_at(sess, dt)
-        if era is None:
-            era_coeff = None
-        else:
-            era_properties = PropDict(
-                chellow.utils.url_root + "eras/" + str(era.id), loads(era.properties)
-            )
-            try:
-                era_coeff = float(era_properties["coefficient"])
-            except KeyError:
-                era_coeff = None
-
         reads = {}
         coeffs = {}
         for coeff, value, tpr_code in sess.query(
@@ -1638,7 +1707,7 @@ def _read_generator(sess, supply, start, is_forwards, is_prev):
             r_dt == dt,
         ):
             reads[tpr_code] = value
-            coeffs[tpr_code] = coeff if era_coeff is None else era_coeff
+            coeffs[tpr_code] = coeff
 
         yield {
             "date": dt,
