@@ -16,20 +16,17 @@ from chellow.utils import (
 )
 
 
-BANDS = ("red", "amber", "green")
+BANDS = ("black", "super-red", "red", "amber", "yellow", "green")
 
-KEYS = dict(
-    (
-        band,
-        {
-            "kwh": f"duos-{band}-kwh",
-            "tariff-rate": f"{band}-gbp-per-kwh",
-            "bill-rate": f"duos-{band}-rate",
-            "gbp": f"duos-{band}-gbp",
-        },
-    )
+KEYS = {
+    band: {
+        "kwh": f"duos-{band}-kwh",
+        "tariff-rate": f"{band}-gbp-per-kwh",
+        "bill-rate": f"duos-{band}-rate",
+        "gbp": f"duos-{band}-gbp",
+    }
     for band in BANDS
-)
+}
 
 VL_LOOKUP = {
     "LV": {True: "lv-sub", False: "lv-net"},
@@ -541,7 +538,7 @@ def datum_2010_04_01(ds, hh):
 
     hh["duos-reactive-kvarh"] = kvarh
 
-    duos_reactive_rate = tariff["gbp-per-kvarh"]
+    duos_reactive_rate = tariff.get("gbp-per-kvarh")
     if duos_reactive_rate is not None:
         duos_reactive_rate = float(duos_reactive_rate)
         if duos_reactive_rate != 0:
@@ -659,34 +656,138 @@ def datum_2012_02_23(ds, hh):
 
             tariffs[start_date] = tariff
 
-    try:
-        band = gsp_group_cache["bands"][start_date]
-    except KeyError:
+    if KEYS["super-red"]["tariff-rate"] in tariff:
         try:
-            bands_cache = gsp_group_cache["bands"]
+            band = gsp_group_cache["super-red-band"][start_date]
         except KeyError:
-            bands_cache = gsp_group_cache["bands"] = {}
-
-        try:
-            band = bands_cache[start_date]
-        except KeyError:
-            band = "green"
-            ct_hr = hh["ct-decimal-hour"]
-            weekend = hh["ct-day-of-week"] > 4
             try:
-                slots = ds.hh_rate(ds.dno_contract.id, start_date)[ds.gsp_group_code][
-                    "bands"
-                ]
-            except KeyError as e:
-                raise BadRequest(str(e))
+                bands_cache = gsp_group_cache["super-red-band"]
+            except KeyError:
+                bands_cache = gsp_group_cache["super-red-band"] = {}
 
-            for slot in slots:
-                slot_weekend = slot["weekend"] == 1
-                if slot_weekend == weekend and slot["start"] <= ct_hr < slot["finish"]:
-                    band = slot["band"]
-                    break
+            try:
+                band = bands_cache[start_date]
+            except KeyError:
+                band = None
+                ct_hr = hh["ct-decimal-hour"]
+                weekend = hh["ct-day-of-week"] > 4
+                try:
+                    slots = ds.hh_rate(ds.dno_contract.id, start_date)[
+                        ds.gsp_group_code
+                    ]["super_red"]
+                except KeyError as e:
+                    raise BadRequest(str(e))
 
-            bands_cache[start_date] = band
+                for slot in slots:
+                    if (
+                        slot["weekend"] == weekend
+                        and slot["start-hour"] <= ct_hr < slot["finish-hour"]
+                        and (
+                            hh["ct-month"] > slot["start-month"]
+                            or (
+                                hh["ct-month"] == slot["start-month"]
+                                and hh["ct-day"] >= slot["start-day"]
+                            )
+                        )
+                        and (
+                            hh["ct-month"] < slot["finish-month"]
+                            or (
+                                hh["ct-month"] == slot["finish-month"]
+                                and (
+                                    slot["finish-day"] == "last"
+                                    or hh["ct-day"] <= slot["finish-day"]
+                                )
+                            )
+                        )
+                    ):
+                        band = "super-red"
+                        break
+
+                bands_cache[start_date] = band
+    elif ds.dtc_meter_type_code is None:  # unmetered
+        try:
+            band = gsp_group_cache["ums-bands"][start_date]
+        except KeyError:
+            try:
+                bands_cache = gsp_group_cache["ums-bands"]
+            except KeyError:
+                bands_cache = gsp_group_cache["ums-bands"] = {}
+
+            try:
+                band = bands_cache[start_date]
+            except KeyError:
+                band = "green"
+                ct_hr = hh["ct-decimal-hour"]
+                ct_day = hh["ct-day"]
+                ct_month = hh["ct-month"]
+                weekend = hh["ct-day-of-week"] > 4
+                try:
+                    slots = ds.hh_rate(ds.dno_contract.id, start_date)[
+                        ds.gsp_group_code
+                    ]["ums_bands"]
+                except KeyError as e:
+                    raise BadRequest(str(e))
+
+                for slot in slots:
+                    if (
+                        slot["weekend"] == weekend
+                        and slot["start-decimal-hour"]
+                        <= ct_hr
+                        < slot["finish-decimal-hour"]
+                        and (
+                            ct_month > slot["start-month"]
+                            or (
+                                ct_month == slot["start-month"]
+                                and ct_day >= slot["start-day"]
+                            )
+                        )
+                        and (
+                            ct_month < slot["finish-month"]
+                            or (
+                                ct_month == slot["finish-month"]
+                                and (
+                                    slot["finish-day"] == "last"
+                                    or ct_day <= slot["finish-day"]
+                                )
+                            )
+                        )
+                    ):
+                        band = slot["band"]
+                        break
+
+                bands_cache[start_date] = band
+    else:
+
+        try:
+            band = gsp_group_cache["rag-bands"][start_date]
+        except KeyError:
+            try:
+                bands_cache = gsp_group_cache["rag-bands"]
+            except KeyError:
+                bands_cache = gsp_group_cache["rag-bands"] = {}
+
+            try:
+                band = bands_cache[start_date]
+            except KeyError:
+                band = "green"
+                ct_hr = hh["ct-decimal-hour"]
+                weekend = hh["ct-day-of-week"] > 4
+                try:
+                    slots = ds.hh_rate(ds.dno_contract.id, start_date)[
+                        ds.gsp_group_code
+                    ]["bands"]
+                except KeyError as e:
+                    raise BadRequest(str(e))
+
+                for slot in slots:
+                    if (
+                        slot["weekend"] == weekend
+                        and slot["start"] <= ct_hr < slot["finish"]
+                    ):
+                        band = slot["band"]
+                        break
+
+                bands_cache[start_date] = band
 
     try:
         laf = dno_cache["lafs"][ds.llfc_code][start_date]
@@ -752,17 +853,18 @@ def datum_2012_02_23(ds, hh):
 
     hh["duos-description"] = tariff["description"]
 
-    duos_reactive_rate = tariff["gbp-per-kvarh"]
+    duos_reactive_rate = tariff.get("gbp-per-kvarh")
     if duos_reactive_rate is not None:
         duos_reactive_rate = float(duos_reactive_rate)
         if duos_reactive_rate != 0:
             hh["duos-reactive-rate"] = duos_reactive_rate
             hh["duos-reactive-gbp"] = kvarh * duos_reactive_rate
 
-    rate = float(tariff[KEYS[band]["tariff-rate"]])
-    hh[KEYS[band]["bill-rate"]] = rate
-    hh[KEYS[band]["kwh"]] = hh["msp-kwh"]
-    hh[KEYS[band]["gbp"]] = rate * hh["msp-kwh"]
+    if band is not None:
+        rate = float(tariff[KEYS[band]["tariff-rate"]])
+        hh[KEYS[band]["bill-rate"]] = rate
+        hh[KEYS[band]["kwh"]] = hh["msp-kwh"]
+        hh[KEYS[band]["gbp"]] = rate * hh["msp-kwh"]
 
     if hh["ct-decimal-hour"] == 23.5 and not ds.is_displaced:
         hh["duos-fixed-days"] = 1
@@ -784,14 +886,7 @@ def datum_2012_02_23(ds, hh):
         days_in_month = 0
         for dsc in ds.get_data_sources(month_from, month_to):
             for datum in dsc.hh_data:
-                md_kva = max(
-                    md_kva,
-                    (
-                        datum["msp-kw"] ** 2
-                        + max(datum["imp-msp-kvar"], datum["exp-msp-kvar"]) ** 2
-                    )
-                    ** 0.5,
-                )
+                md_kva = max(md_kva, datum["msp-kva"])
                 if datum["ct-decimal-hour"] == 0:
                     days_in_month += 1
 
