@@ -6,14 +6,15 @@ import tempfile
 import threading
 import traceback
 from collections import defaultdict, deque
-from datetime import timedelta as Timedelta
-from io import TextIOWrapper
+from datetime import datetime as Datetime, timedelta as Timedelta
+from decimal import Decimal
+from io import StringIO
+from pathlib import Path
 
 import jinja2
 
-import paramiko
+from paramiko import AutoAddPolicy, RSAKey, SFTPError, SSHClient
 
-import pysftp
 
 import requests
 
@@ -24,9 +25,13 @@ from werkzeug.exceptions import BadRequest
 from chellow.models import Contract, Era, HhDatum, MarketRole, Session
 from chellow.utils import (
     HH,
+    ct_datetime,
+    ct_datetime_now,
     hh_format,
     hh_max,
     hh_min,
+    to_ct,
+    to_utc,
     utc_datetime,
     utc_datetime_now,
 )
@@ -35,7 +40,15 @@ from chellow.utils import (
 processes = defaultdict(list)
 tasks = {}
 
-extensions = [".df2", ".simple.csv", ".bg.csv"]
+extensions = [
+    ".df2",
+    ".simple.csv",
+    ".bg.csv",
+    ".vital.xlsx",
+    ".edf.csv",
+    ".schneider.csv",
+    ".schneider.xlsx",
+]
 
 
 class HhDataImportProcess(threading.Thread):
@@ -43,6 +56,7 @@ class HhDataImportProcess(threading.Thread):
         super().__init__(name=f"HH Manual Import: contract {dc_contract_id}")
 
         self.messages = []
+        self.error = None
         self.istream = istream
         self.dc_contract_id = dc_contract_id
         self.id = process_id
@@ -69,14 +83,16 @@ class HhDataImportProcess(threading.Thread):
                 parser_module = importlib.import_module(
                     f"chellow.e.hh_parser_{mod_name}"
                 )
-                self.converter = parser_module.create_parser(self.istream, mpan_map)
+                self.converter = parser_module.create_parser(
+                    self.istream, mpan_map, self.messages
+                )
                 sess.rollback()
                 HhDatum.insert(sess, self.converter, contract)
                 sess.commit()
             except BadRequest as e:
-                self.messages.append(e.description)
+                self.error = e.description
             except BaseException:
-                self.messages.append(f"Outer problem {traceback.format_exc()}")
+                self.error = f"Outer problem {traceback.format_exc()}"
 
     def get_status(self):
         return (
@@ -166,7 +182,7 @@ class HhImportTask(threading.Thread):
             self.log(f"Problem {e}")
             sess.rollback()
             self.is_error = True
-        except Exception:
+        except BaseException:
             self.log(f"Unknown Exception {traceback.format_exc()}")
             sess.rollback()
             self.is_error = True
@@ -185,7 +201,7 @@ class HhImportTask(threading.Thread):
                     lock.release()
 
     def run(self):
-        while not self.stopped.isSet():
+        while not self.stopped.is_set():
             self.import_now()
             timeout = None if self.is_error else self.wait_seconds
             self.going.wait(timeout=timeout)
@@ -223,19 +239,19 @@ class HhImportTask(threading.Thread):
         fl = None
 
         for directory in directories:
-            self.log("Checking the directory '" + directory + "'.")
+            self.log(f"Checking the directory '{directory}'.")
             try:
                 last_import_key = last_import_keys[directory]
             except KeyError:
                 last_import_key = last_import_keys[directory] = ""
 
             dir_path = home_path + "/" + directory
-            ftp.cwd(dir_path)
+            ftp.chdir(dir_path)
             files = []
-            for fname in ftp.nlst():
+            for fname in ftp.listdir():
                 fpath = dir_path + "/" + fname
                 try:
-                    ftp.cwd(fpath)
+                    ftp.chdir(fpath)
                     continue  # directory
                 except ftplib.error_perm:
                     pass
@@ -258,7 +274,7 @@ class HhImportTask(threading.Thread):
             key, fpath = fl
             self.log(f"Attempting to download {fpath} with key {key}.")
             f = tempfile.TemporaryFile()
-            ftp.retrbinary("RETR " + fpath, f.write)
+            ftp.retrbinary(f"RETR {fpath}", f.write)
             self.log("File downloaded successfully.")
             ftp.quit()
             self.log("Logged out.")
@@ -269,32 +285,37 @@ class HhImportTask(threading.Thread):
             self.log(f"File size is {fsize} bytes.")
             self.log(f"Treating files as type {file_type}")
             self.importer = HhDataImportProcess(
-                self.contract_id, 0, TextIOWrapper(f, "utf8"), fpath + file_type, fsize
+                self.contract_id, 0, f, fpath + file_type, fsize
             )
 
             self.importer.run()
             for message in self.importer.messages:
                 self.log(message)
 
-            if len(self.importer.messages) > 0:
-                raise BadRequest("Problem loading file.")
+            if self.importer.error is not None:
+                raise BadRequest(f"Problem loading file. {self.importer.error}")
 
             contract = Contract.get_dc_by_id(sess, self.contract_id)
             contract.update_state(state)
             sess.commit()
-            self.log("Finished loading '" + fpath)
+            self.log(f"Finished loading '{fpath}'")
             return True
 
     def sftp_handler(self, sess, properties, contract):
-        host_name = properties["hostname"]
-        user_name = properties["username"]
-        password = properties["password"]
-        try:
-            port = properties["port"]
-        except KeyError:
-            port = None
+        hostname = properties["hostname"]
+        username = properties["username"]
+        password = properties.get("password")
+        private_key = properties.get("private_key")
+        pkey = None
+        if private_key is not None:
+            pkf = StringIO(private_key)
+            pkf.seek(0)
+            pkey = RSAKey.from_private_key(pkf)
+
+        port = properties.get("port", 22)
         file_type = properties["file_type"]
         directories = properties["directories"]
+        known_hosts = properties["known_hosts"].strip()
         state = contract.make_state()
 
         try:
@@ -302,37 +323,73 @@ class HhImportTask(threading.Thread):
         except KeyError:
             last_import_keys = state["last_import_keys"] = {}
 
+        ct_now = ct_datetime_now()
+        try:
+            latest_imports = [
+                (dt, fname)
+                for dt, fname in state["latest_imports"]
+                if (to_ct(dt).year, to_ct(dt).month, to_ct(dt).day)
+                == (ct_now.year, ct_now.month, ct_now.day)
+            ]
+            state["latest_imports"] = latest_imports
+        except KeyError:
+            latest_imports = state["latest_imports"] = []
+
         sess.rollback()
-        self.log("Connecting to sftp server at " + host_name + ":" + str(port) + ".")
-        cnopts = pysftp.CnOpts()
-        cnopts.hostkeys = None
-        ftp = pysftp.Connection(
-            host_name, username=user_name, password=password, cnopts=cnopts
+        self.log(f"Connecting to sftp server at {hostname}:{port}.")
+
+        client = SSHClient()
+        client.set_missing_host_key_policy(AutoAddPolicy)
+
+        client.connect(
+            hostname,
+            port=port,
+            username=username,
+            password=password,
+            pkey=pkey,
+            key_filename=None,
+            timeout=120,
+            banner_timeout=120,
+            auth_timeout=120,
+            channel_timeout=120,
+            look_for_keys=False,
         )
-        ftp.timeout = 120
-        home_path = ftp.pwd
+        host_keys = client.get_host_keys()
+        with tempfile.TemporaryDirectory() as tempdir:
+            td = Path(tempdir)
+            hfname = td / "hosts"
+            host_keys.save(hfname)
+            with hfname.open() as hf:
+                host_keys_str = hf.read().strip()
+
+        if host_keys_str != known_hosts:
+            raise BadRequest(
+                f"The returned host keys {host_keys_str} don't match the known "
+                f"hosts {known_hosts} property"
+            )
+
+        ftp = client.open_sftp()
 
         f = None
 
         for directory in directories:
-            self.log("Checking the directory '" + directory + "'.")
+            self.log(f"Checking the directory '{directory}'.")
             try:
                 last_import_key = last_import_keys[directory]
             except KeyError:
                 last_import_key = last_import_keys[directory] = ""
 
-            dir_path = home_path + "/" + directory
-            ftp.cwd(dir_path)
+            ftp.chdir(directory)
             files = []
             for attr in ftp.listdir_attr():
-                fpath = dir_path + "/" + attr.filename
+                fpath = f"{directory}/{attr.filename}"
                 try:
-                    ftp.cwd(fpath)
+                    ftp.chdir(fpath)
                     continue  # directory
-                except paramiko.SFTPError:
+                except SFTPError:
                     pass
 
-                key = str(attr.st_mtime) + "_" + attr.filename
+                key = f"{attr.st_mtime}_{attr.filename}"
                 if key > last_import_key:
                     files.append((key, fpath))
 
@@ -344,11 +401,13 @@ class HhImportTask(threading.Thread):
         if f is None:
             self.log("No new files found.")
             ftp.close()
+            client.close()
             self.log("Logged out.")
             return False
         else:
             key, fpath = f
-            self.log("Attempting to download " + fpath + " with key " + key + ".")
+            latest_imports.append((utc_datetime_now(), fpath))
+            self.log(f"Attempting to download {fpath} with key {key}.")
             f = tempfile.TemporaryFile()
             ftp.getfo(fpath, f)
             self.log("File downloaded successfully.")
@@ -358,33 +417,164 @@ class HhImportTask(threading.Thread):
             f.seek(0, os.SEEK_END)
             fsize = f.tell()
             f.seek(0)
-            self.log("File size is " + str(fsize) + " bytes.")
-            self.log("Treating files as type " + file_type)
+            self.log(f"File size is {fsize} bytes.")
+            self.log(f"Treating files as type {file_type}")
             self.importer = HhDataImportProcess(
-                self.contract_id, 0, TextIOWrapper(f, "utf8"), fpath + file_type, fsize
+                self.contract_id, 0, f, fpath + file_type, fsize
             )
 
             self.importer.run()
             messages = self.importer.messages
-            self.importer = None
             for message in messages:
                 self.log(message)
 
-            if len(messages) > 0:
-                raise BadRequest("Problem loading file.")
+            if self.importer.error is not None:
+                raise BadRequest(f"Problem loading file. {self.importer.error}")
+            self.importer = None
 
             contract = Contract.get_dc_by_id(sess, self.contract_id)
             contract.update_state(state)
             sess.commit()
-            self.log("Finished loading '" + fpath)
+            self.log(f"Finished loading '{fpath}'")
             return True
 
 
+def meniscus_parser(log_f, result, properties, mpan_core):
+    if isinstance(result, dict):
+        result_data = result["Data"]
+    elif isinstance(result, list):
+        result_data = result
+    else:
+        raise BadRequest(
+            f"Expecting a JSON object at the top level, but instead got {result}"
+        )
+    raw_data = []
+    for jdatum in result_data:
+        raw_data.append(
+            dict(
+                mpan_core=mpan_core,
+                start_date=utc_datetime(1, 1, 1)
+                + Timedelta(seconds=jdatum["Time"] / 10000000),
+                channel_type="ACTIVE",
+                value=jdatum["Value"],
+                status="A",
+            )
+        )
+    return raw_data
+
+
+def solaredge_meters_parser(log_f, result, properties, mpan_core):
+    meters = result["meterEnergyDetails"]["meters"]
+    meter = None
+    if mpan_core in properties["generated_mpan_cores"]:
+        meter_type = "FeedIn"
+    elif mpan_core in properties["parasitic_mpan_cores"]:
+        meter_type = "Purchased"
+    else:
+        raise BadRequest(
+            f"Can't find the MPAN core {mpan_core} in either the generated_mpan_cores "
+            f"or parasitic_mpan_cores lists in the DC properties."
+        )
+    for m in meters:
+        if m["meterType"] == meter_type:
+            meter = m
+            break
+
+    if meter is None:
+        raise BadRequest(
+            f"Can't find the meter type '{meter_type}' in the list of meters."
+        )
+
+    raw_data = {}
+    prev_read = None
+    for val in meter["values"]:
+        if "value" in val:
+            d_ct = to_ct(Datetime.strptime(val["date"], "%Y-%m-%d %H:%M:%S"))
+            read = Decimal(val["value"])
+            if prev_read is not None:
+                value = (read - prev_read) / Decimal("1000")
+                if d_ct.minute > 30:
+                    date_ct_minute = 30
+                else:
+                    date_ct_minute = 0
+
+                date_ct = ct_datetime(
+                    d_ct.year, d_ct.month, d_ct.day, d_ct.hour, date_ct_minute
+                )
+
+                try:
+                    raw_datum = raw_data[date_ct]
+                except KeyError:
+                    raw_datum = raw_data[date_ct] = {
+                        "mpan_core": mpan_core,
+                        "start_date": to_utc(date_ct),
+                        "channel_type": "ACTIVE",
+                        "value": Decimal(0),
+                        "status": "A",
+                    }
+
+                raw_datum["value"] += value
+            prev_read = read
+
+    return [v for _, v in sorted(raw_data.items())]
+
+
+def solaredge_energy_details_parser(log_f, result, properties, mpan_core):
+    meters = result["energyDetails"]["meters"]
+    meter = None
+    if mpan_core in properties["generated_mpan_cores"]:
+        meter_key = "Production"
+    elif mpan_core in properties["parasitic_mpan_cores"]:
+        meter_key = "Consumption"
+    else:
+        raise BadRequest(
+            f"Can't find the MPAN core {mpan_core} in either the generated_mpan_cores "
+            f"or parasitic_mpan_cores lists in the DC properties."
+        )
+    for m in meters:
+        if m["type"] == meter_key:
+            meter = m
+            break
+
+    if meter is None:
+        raise BadRequest(
+            f"Can't find the meter type '{meter_key}' in the list of meters."
+        )
+
+    raw_data = []
+    for val in meter["values"]:
+        if "value" in val:
+            date_ct = to_ct(Datetime.strptime(val["date"], "%Y-%m-%d %H:%M:%S"))
+            value = Decimal(val["value"]) / Decimal("1000")
+            if date_ct.minute in (0, 30):
+                raw_data.append(
+                    {
+                        "mpan_core": mpan_core,
+                        "start_date": to_utc(date_ct),
+                        "channel_type": "ACTIVE",
+                        "value": value,
+                        "status": "A",
+                    }
+                )
+            elif len(raw_data) > 0:
+                raw_data[-1]["value"] += value
+
+    return raw_data
+
+
+PARSERS = {
+    "meniscus": meniscus_parser,
+    "solaredge_energy_details": solaredge_energy_details_parser,
+    "solaredge_meters": solaredge_meters_parser,
+}
+
+
 def https_handler(sess, log_f, properties, contract, now=None):
     url_template_str = properties["url_template"]
     url_values = properties.get("url_values", {})
     download_days = properties["download_days"]
-    result_data_key = properties["result_data_key"]
+    parser_name = properties["parser"]
+
     if now is None:
         now = utc_datetime_now()
     window_finish = utc_datetime(now.year, now.month, now.day) - HH
@@ -426,30 +616,13 @@ def https_handler(sess, log_f, properties, contract, now=None):
             log_f(f"Retrieving data from {url}.")
 
             sess.rollback()  # Avoid long transactions
-            res = requests.get(url, timeout=120)
+
+            s = requests.Session()
+            s.verify = False
+            res = s.get(url, timeout=120)
             res.raise_for_status()
-            result = res.json()
-            if isinstance(result, dict):
-                result_data = result[result_data_key]
-            elif isinstance(result, list):
-                result_data = result
-            else:
-                raise BadRequest(
-                    f"Expecting a JSON object at the top level, but instead got "
-                    f"{result}"
-                )
-            raw_data = []
-            for jdatum in result_data:
-                raw_data.append(
-                    dict(
-                        mpan_core=mpan_core,
-                        start_date=utc_datetime(1, 1, 1)
-                        + Timedelta(seconds=jdatum["Time"] / 10000000),
-                        channel_type="ACTIVE",
-                        value=jdatum["Value"],
-                        status="A",
-                    )
-                )
+
+            raw_data = PARSERS[parser_name](log_f, res.json(), properties, mpan_core)
             HhDatum.insert(sess, raw_data, contract)
             sess.commit()
     log_f("Finished loading.")
