@@ -15,13 +15,16 @@ from chellow.e.computer import (
     contract_func,
     datum_range,
     displaced_era,
+    hh_rate,
 )
 from chellow.models import Era, Llfc, MtcParticipant, Pc, SiteEra, Source, Supply
 from chellow.utils import (
+    PropDict,
     c_months_u,
     hh_format,
     hh_max,
     hh_min,
+    hh_range,
     parse_hh_start,
     to_ct,
     to_utc,
@@ -83,21 +86,22 @@ def make_create_future_func_monthly(contract_name, fnames):
 def make_site_deltas(
     sess, report_context, site, scenario_hh, forecast_from, supply_ids
 ):
-    site_scenario_hh = scenario_hh.get(site.code, {})
+    site_scenario_hh_str = scenario_hh.get(site.code, {})
+    site_scenario_hh = {}
 
     site_deltas = {"hhs": {}}
     delts = site_deltas["supply_deltas"] = {}
     for is_import in (True, False):
         delts[is_import] = {}
-        for src in ("gen", "net", "gen-net", "3rd-party", "3rd-party-reverse", "sub"):
+        for src in ("gen", "grid", "gen-grid", "3rd-party", "3rd-party-reverse", "sub"):
             delts[is_import][src] = {"site": {}}
 
     earliest_delta = to_utc(Datetime.max)
     latest_delta = to_utc(Datetime.min)
 
     found_hh = False
-    for typ in ("used", "generated", "parasitic", "gen_net"):
-        hh_str = site_scenario_hh.get(typ, "")
+    for typ in ("used", "generated", "parasitic", "gen_grid"):
+        hh_str = site_scenario_hh_str.get(typ, "")
         hh_data = site_scenario_hh[typ] = {}
         for row in csv.reader(StringIO(hh_str)):
             cells = [cell.strip() for cell in row]
@@ -129,7 +133,7 @@ def make_site_deltas(
     scenario_used = site_scenario_hh["used"]
     scenario_generated = site_scenario_hh["generated"]
     scenario_parasitic = site_scenario_hh["parasitic"]
-    scenario_gen_net = site_scenario_hh["gen_net"]
+    scenario_gen_grid = site_scenario_hh["gen_grid"]
 
     earliest_delta_ct = to_ct(earliest_delta)
     for month_start, month_finish in c_months_u(
@@ -170,7 +174,7 @@ def make_site_deltas(
 
             for hh in ss.hh_data:
                 sdatum = hh_map[hh["start-date"]]
-                sdatum["import-net-kwh"] += hh["msp-kwh"]
+                sdatum["import-grid-kwh"] += hh["msp-kwh"]
                 sdatum["used-kwh"] += hh["msp-kwh"]
 
         for era in (
@@ -185,7 +189,7 @@ def make_site_deltas(
                 Era.imp_mpan_core != null(),
                 Era.start_date <= chunk_finish,
                 or_(Era.finish_date == null(), Era.finish_date >= chunk_start),
-                Source.code == "gen-net",
+                Source.code == "gen-grid",
             )
         ):
             if supply_ids is not None and era.supply_id not in supply_ids:
@@ -201,65 +205,65 @@ def make_site_deltas(
             for hh in ss.hh_data:
                 sdatum = hh_map[hh["start-date"]]
                 try:
-                    sdatum["gen-net-kwh"] += hh["msp-kwh"]
+                    sdatum["gen-grid-kwh"] += hh["msp-kwh"]
                 except KeyError:
-                    sdatum["gen-net-kwh"] = hh["msp-kwh"]
+                    sdatum["gen-grid-kwh"] = hh["msp-kwh"]
 
         for hh_start, hh in hh_map.items():
             if hh_start in scenario_used:
                 used_delt = scenario_used[hh_start] - hh["used-kwh"]
-                imp_net_delt = 0
-                exp_net_delt = 0
+                imp_grid_delt = 0
+                exp_grid_delt = 0
 
                 if used_delt < 0:
-                    diff = hh["import-net-kwh"] + used_delt
+                    diff = hh["import-grid-kwh"] + used_delt
                     if diff < 0:
-                        imp_net_delt -= hh["import-net-kwh"]
-                        exp_net_delt -= diff
+                        imp_grid_delt -= hh["import-grid-kwh"]
+                        exp_grid_delt -= diff
                     else:
-                        imp_net_delt += used_delt
+                        imp_grid_delt += used_delt
                 else:
-                    diff = hh["export-net-kwh"] - used_delt
+                    diff = hh["export-grid-kwh"] - used_delt
                     if diff < 0:
-                        exp_net_delt -= hh["export-net-kwh"]
-                        imp_net_delt -= diff
+                        exp_grid_delt -= hh["export-grid-kwh"]
+                        imp_grid_delt -= diff
                     else:
-                        exp_net_delt -= used_delt
+                        exp_grid_delt -= used_delt
 
                 try:
-                    delts[False]["net"]["site"][hh_start] += exp_net_delt
+                    delts[False]["grid"]["site"][hh_start] += exp_grid_delt
                 except KeyError:
-                    delts[False]["net"]["site"][hh_start] = exp_net_delt
+                    delts[False]["grid"]["site"][hh_start] = exp_grid_delt
 
                 try:
-                    delts[True]["net"]["site"][hh_start] += imp_net_delt
+                    delts[True]["grid"]["site"][hh_start] += imp_grid_delt
                 except KeyError:
-                    delts[True]["net"]["site"][hh_start] = imp_net_delt
+                    delts[True]["grid"]["site"][hh_start] = imp_grid_delt
 
-                hh["import-net-kwh"] += imp_net_delt
-                hh["export-net-kwh"] += exp_net_delt
+                hh["import-grid-kwh"] += imp_grid_delt
+                hh["export-grid-kwh"] += exp_grid_delt
                 hh["used-kwh"] += used_delt
-                hh["msp-kwh"] -= exp_net_delt
+                hh["msp-kwh"] -= exp_grid_delt
 
             if hh_start in scenario_generated:
                 imp_gen_delt = scenario_generated[hh_start] - hh["import-gen-kwh"]
-                imp_net_delt = 0
-                exp_net_delt = 0
+                imp_grid_delt = 0
+                exp_grid_delt = 0
 
                 if imp_gen_delt < 0:
-                    diff = hh["export-net-kwh"] + imp_gen_delt
+                    diff = hh["export-grid-kwh"] + imp_gen_delt
                     if diff < 0:
-                        exp_net_delt -= hh["export-net-kwh"]
-                        imp_net_delt -= diff
+                        exp_grid_delt -= hh["export-grid-kwh"]
+                        imp_grid_delt -= diff
                     else:
-                        exp_net_delt += imp_gen_delt
+                        exp_grid_delt += imp_gen_delt
                 else:
-                    diff = hh["import-net-kwh"] - imp_gen_delt
+                    diff = hh["import-grid-kwh"] - imp_gen_delt
                     if diff < 0:
-                        imp_net_delt -= hh["import-net-kwh"]
-                        exp_net_delt -= diff
+                        imp_grid_delt -= hh["import-grid-kwh"]
+                        exp_grid_delt -= diff
                     else:
-                        imp_net_delt -= imp_gen_delt
+                        imp_grid_delt -= imp_gen_delt
 
                 try:
                     delts[True]["gen"]["site"][hh_start] += imp_gen_delt
@@ -267,39 +271,39 @@ def make_site_deltas(
                     delts[True]["gen"]["site"][hh_start] = imp_gen_delt
 
                 try:
-                    delts[False]["net"]["site"][hh_start] += exp_net_delt
+                    delts[False]["grid"]["site"][hh_start] += exp_grid_delt
                 except KeyError:
-                    delts[False]["net"]["site"][hh_start] = exp_net_delt
+                    delts[False]["grid"]["site"][hh_start] = exp_grid_delt
 
                 try:
-                    delts[True]["net"]["site"][hh_start] += imp_net_delt
+                    delts[True]["grid"]["site"][hh_start] += imp_grid_delt
                 except KeyError:
-                    delts[True]["net"]["site"][hh_start] = imp_net_delt
+                    delts[True]["grid"]["site"][hh_start] = imp_grid_delt
 
-                hh["import-net-kwh"] += imp_net_delt
-                hh["export-net-kwh"] += exp_net_delt
+                hh["import-grid-kwh"] += imp_grid_delt
+                hh["export-grid-kwh"] += exp_grid_delt
                 hh["import-gen-kwh"] += imp_gen_delt
-                hh["msp-kwh"] -= imp_net_delt
+                hh["msp-kwh"] -= imp_grid_delt
 
             if hh_start in scenario_parasitic:
                 exp_gen_delt = scenario_parasitic[hh_start] - hh["export-gen-kwh"]
-                imp_net_delt = 0
-                exp_net_delt = 0
+                imp_grid_delt = 0
+                exp_grid_delt = 0
 
                 if exp_gen_delt < 0:
-                    diff = hh["import-net-kwh"] + exp_gen_delt
+                    diff = hh["import-grid-kwh"] + exp_gen_delt
                     if diff < 0:
-                        imp_net_delt -= hh["import-net-kwh"]
-                        exp_net_delt -= diff
+                        imp_grid_delt -= hh["import-grid-kwh"]
+                        exp_grid_delt -= diff
                     else:
-                        imp_net_delt += exp_gen_delt
+                        imp_grid_delt += exp_gen_delt
                 else:
-                    diff = hh["export-net-kwh"] - exp_gen_delt
+                    diff = hh["export-grid-kwh"] - exp_gen_delt
                     if diff < 0:
-                        exp_net_delt -= hh["export-net-kwh"]
-                        imp_net_delt -= diff
+                        exp_grid_delt -= hh["export-grid-kwh"]
+                        imp_grid_delt -= diff
                     else:
-                        exp_net_delt -= exp_gen_delt
+                        exp_grid_delt -= exp_gen_delt
 
                 try:
                     delts[False]["gen"]["site"][hh_start] += imp_gen_delt
@@ -307,34 +311,34 @@ def make_site_deltas(
                     delts[False]["gen"]["site"][hh_start] = exp_gen_delt
 
                 try:
-                    delts[False]["net"]["site"][hh_start] += exp_net_delt
+                    delts[False]["grid"]["site"][hh_start] += exp_grid_delt
                 except KeyError:
-                    delts[False]["net"]["site"][hh_start] = exp_net_delt
+                    delts[False]["grid"]["site"][hh_start] = exp_grid_delt
 
                 try:
-                    delts[True]["net"]["site"][hh_start] += imp_net_delt
+                    delts[True]["grid"]["site"][hh_start] += imp_grid_delt
                 except KeyError:
-                    delts[True]["net"]["site"][hh_start] = imp_net_delt
+                    delts[True]["grid"]["site"][hh_start] = imp_grid_delt
 
-                hh["import-net-kwh"] += imp_net_delt
-                hh["export-net-kwh"] += exp_net_delt
+                hh["import-grid-kwh"] += imp_grid_delt
+                hh["export-grid-kwh"] += exp_grid_delt
                 hh["export-gen-kwh"] += exp_gen_delt
-                hh["msp-kwh"] -= imp_net_delt
+                hh["msp-kwh"] -= imp_grid_delt
 
-            if hh_start in scenario_gen_net:
-                gen_net_delt = scenario_gen_net[hh_start] - hh["gen-net-kwh"]
+            if hh_start in scenario_gen_grid:
+                gen_grid_delt = scenario_gen_grid[hh_start] - hh["gen-grid-kwh"]
 
                 try:
-                    delts[False]["gen-net"]["site"][hh_start] += gen_net_delt
+                    delts[False]["gen-grid"]["site"][hh_start] += gen_grid_delt
                 except KeyError:
-                    delts[False]["gen-net"]["site"][hh_start] = gen_net_delt
+                    delts[False]["gen-grid"]["site"][hh_start] = gen_grid_delt
 
-                hh["import-gen-kwh"] += gen_net_delt
-                hh["export-net-kwh"] += gen_net_delt
+                hh["import-gen-kwh"] += gen_grid_delt
+                hh["export-grid-kwh"] += gen_grid_delt
 
             site_deltas["hhs"][hh_start] = hh
 
-    sup_deltas = site_deltas["supply_deltas"][False]["net"]["site"]
+    sup_deltas = site_deltas["supply_deltas"][False]["grid"]["site"]
     if all(v == 0 for v in sup_deltas.values()):
         sup_deltas.clear()
 
@@ -381,6 +385,7 @@ def make_calcs(
             joinedload(Era.supply).joinedload(Supply.source),
             joinedload(Era.mtc_participant).joinedload(MtcParticipant.meter_type),
             joinedload(Era.pc),
+            joinedload(Era.energisation_status),
             joinedload(Era.site_eras),
         )
         .order_by(Era.supply_id, Era.start_date)
@@ -478,7 +483,6 @@ def make_calcs(
             f"{meter_order[measurement_type]}_{supply.id}_{hh_format(era.start_date)}"
         )
         calcs.append((order, era.imp_mpan_core, era.exp_mpan_core, imp_ss, exp_ss))
-        sess.rollback()
 
     start_date_ct, finish_date_ct = to_ct(start_date), to_ct(finish_date)
     for month_start, month_finish in c_months_u(
@@ -531,7 +535,7 @@ def make_calcs(
                 )
             disp_vb_function(site_ds)
 
-            calcs.append(("1", "displaced", None, site_ds, None))
+            calcs.append((f"1_{hh_format(ss_start)}", "displaced", None, site_ds, None))
 
         if len(extra_sss) > 0:
             if True in extra_sss:
@@ -569,10 +573,10 @@ def make_calcs(
 
             calcs.append(("0", imp_ss_name, exp_ss_name, imp_ss, exp_ss))
 
-        # Check if exp net deltas haven't been consumed
-        sup_deltas = site_deltas["supply_deltas"][False]["net"]
+        # Check if exp grid deltas haven't been consumed
+        sup_deltas = site_deltas["supply_deltas"][False]["grid"]
         if len(list(t for t in sup_deltas["site"] if ss_start <= t <= ss_finish)) > 0:
-            ss_name = site.code + "_extra_net_export"
+            ss_name = site.code + "_extra_grid_export"
             ss = SupplySource(
                 sess,
                 ss_start,
@@ -587,7 +591,6 @@ def make_calcs(
             )
 
             calcs.append(("0", None, ss_name, None, ss))
-        sess.rollback()
     return calcs, site_gen_types
 
 
@@ -661,3 +664,46 @@ class ScenarioSource:
 
                 hh["msp-kwh"] += delt
                 hh["msp-kw"] += delt * 2
+
+
+def scenario_fill_cache(report_context, sess, scenario_props):
+    try:
+        comp = report_context["computer"]
+    except KeyError:
+        comp = report_context["computer"] = {}
+
+    try:
+        rate_cache = comp["rates"]
+    except KeyError:
+        rate_cache = comp["rates"] = {}
+
+    for rate_script in scenario_props.get("rates", []):
+        contract_id = rate_script["contract_id"]
+        try:
+            cont_cache = rate_cache[contract_id]
+        except KeyError:
+            cont_cache = rate_cache[contract_id] = {}
+
+        try:
+            rate_script_start = rate_script["start_date"]
+        except KeyError:
+            raise BadRequest(
+                f"Problem in the scenario properties. Can't find the 'start_date' key "
+                f"of the contract {contract_id} in the 'rates' map."
+            )
+
+        try:
+            rate_script_finish = rate_script["finish_date"]
+        except KeyError:
+            raise BadRequest(
+                f"Problem in the scenario properties. Can't find the 'finish_date' "
+                f"key of the contract {contract_id} in the 'rates' map."
+            )
+
+        for dt in hh_range(report_context, rate_script_start, rate_script_finish):
+            hh_rate(sess, report_context, contract_id, dt)
+
+        for dt in hh_range(report_context, rate_script_start, rate_script_finish):
+            storage = cont_cache[dt]._storage.copy()
+            storage.update(rate_script["script"])
+            cont_cache[dt] = PropDict("scenario properties", storage)
