@@ -4,12 +4,12 @@ import threading
 import traceback
 from decimal import Decimal
 
-from sqlalchemy import null, or_
+from sqlalchemy import null, or_, select
 from sqlalchemy.sql.expression import false
 
 from werkzeug.exceptions import BadRequest
 
-from zish import ZishException, loads
+from zish import loads
 
 from chellow.models import (
     Bill,
@@ -18,9 +18,12 @@ from chellow.models import (
     Comm,
     Contract,
     Cop,
+    DtcMeterType,
     EnergisationStatus,
     Era,
+    GBill,
     GContract,
+    GEra,
     GExitZone,
     GReadType,
     GReadingFrequency,
@@ -81,7 +84,8 @@ ALLOWED_ACTIONS = ("insert", "update", "delete")
 
 def general_import_era(sess, action, vals, args):
     if action == "update":
-        mpan_core = add_arg(args, "mpan_core", vals, 0)
+        mpan_core_str = add_arg(args, "mpan_core", vals, 0)
+        mpan_core = parse_mpan_core(mpan_core_str)
         supply = Supply.get_by_mpan_core(sess, mpan_core)
         date_str = add_arg(args, "date", vals, 1)
         dt = parse_hh_start(date_str)
@@ -146,13 +150,13 @@ def general_import_era(sess, action, vals, args):
         comm_code = add_arg(args, "Comms Type", vals, 12)
         comm = era.comm if comm_code == NO_CHANGE else Comm.get_by_code(sess, comm_code)
 
-        ssc_code = add_arg(args, "SSC", vals, 13)
-        if ssc_code == NO_CHANGE:
-            ssc = era.ssc
-        elif len(ssc_code) == 0:
-            ssc = None
+        ssc_code_str = add_arg(args, "SSC", vals, 13)
+        if ssc_code_str == NO_CHANGE:
+            ssc_code = None if era.ssc is None else era.ssc.code
+        elif len(ssc_code_str) == 0:
+            ssc_code = None
         else:
-            ssc = Ssc.get_by_code(sess, ssc_code)
+            ssc_code = ssc_code_str
 
         es_code = add_arg(args, "Energisation Status", vals, 14)
         if es_code == NO_CHANGE:
@@ -160,11 +164,11 @@ def general_import_era(sess, action, vals, args):
         else:
             es = EnergisationStatus.get_by_code(sess, es_code)
 
-        properties = add_arg(args, "Properties", vals, 15)
-        if properties == NO_CHANGE:
-            properties = loads(era.properties)
+        dtc_meter_type_code = add_arg(args, "DTC Meter Type", vals, 15)
+        if dtc_meter_type_code == NO_CHANGE:
+            dtc_meter_type = era.dtc_meter_type
         else:
-            properties = loads(properties)
+            dtc_meter_type = DtcMeterType.get_by_code(sess, dtc_meter_type_code)
 
         imp_mpan_core = add_arg(args, "Import MPAN Core", vals, 16)
         imp_llfc_code = None
@@ -264,9 +268,9 @@ def general_import_era(sess, action, vals, args):
             mtc_code,
             cop,
             comm,
-            ssc,
+            ssc_code,
             es,
-            properties,
+            dtc_meter_type,
             imp_mpan_core,
             imp_llfc_code,
             imp_supplier_contract,
@@ -373,10 +377,14 @@ def general_import_era(sess, action, vals, args):
         ssc_code = add_arg(args, "Standard Settlement Configuration", vals, 12)
         if ssc_code == NO_CHANGE:
             ssc = existing_era.ssc
+            if ssc is None:
+                ssc_code = None
+            else:
+                ssc_code = ssc.code
         elif len(ssc_code) > 0:
-            ssc = Ssc.get_by_code(sess, ssc_code)
+            Ssc.get_by_code(sess, ssc_code, start_date)
         else:
-            ssc = None
+            ssc_code = None
 
         es_code = add_arg(args, "Energisation Status", vals, 13)
         if es_code == NO_CHANGE:
@@ -384,11 +392,11 @@ def general_import_era(sess, action, vals, args):
         else:
             energisation_status = EnergisationStatus.get_by_code(sess, es_code)
 
-        properties_str = add_arg(args, "Properties", vals, 14)
-        if properties_str == NO_CHANGE:
-            properties = loads(existing_era.properties)
+        dtc_meter_type_code = add_arg(args, "DTC Meter Type", vals, 14)
+        if dtc_meter_type_code == NO_CHANGE:
+            dtc_meter_type = existing_era.dtc_meter_type
         else:
-            properties = loads(properties_str)
+            dtc_meter_type = DtcMeterType.get_by_code(sess, dtc_meter_type_code)
 
         imp_mpan_core = add_arg(args, "Import MPAN Core", vals, 15)
         if imp_mpan_core == NO_CHANGE:
@@ -512,9 +520,9 @@ def general_import_era(sess, action, vals, args):
             mtc_code,
             cop,
             comm,
-            ssc,
+            ssc_code,
             energisation_status,
-            properties,
+            dtc_meter_type,
             imp_mpan_core,
             imp_llfc_code,
             imp_supplier_contract,
@@ -556,6 +564,10 @@ def general_import_g_supply(sess, action, vals, args):
         g_reading_frequency = GReadingFrequency.get_by_code(
             sess, g_reading_frequency_code
         )
+        aq_str = add_arg(args, "AQ", vals, 12)
+        aq = Decimal(aq_str)
+        soq_str = add_arg(args, "SOQ", vals, 13)
+        soq = Decimal(soq_str)
 
         site.insert_g_supply(
             sess,
@@ -570,6 +582,8 @@ def general_import_g_supply(sess, action, vals, args):
             g_contract,
             account,
             g_reading_frequency,
+            aq,
+            soq,
         )
         sess.flush()
     elif action == "update":
@@ -589,6 +603,183 @@ def general_import_g_supply(sess, action, vals, args):
         sess.flush()
 
 
+def general_import_g_era(sess, action, vals, args):
+    if action == "insert":
+        mprn = add_arg(args, "MPRN", vals, 0)
+        g_supply = GSupply.get_by_mprn(sess, mprn)
+        start_date_str = add_arg(args, "Start Date", vals, 1)
+        if len(start_date_str) == 0:
+            start_date = None
+        else:
+            start_date = parse_hh_start(start_date_str)
+
+        existing_g_era = sess.scalars(
+            select(GEra).where(
+                GEra.g_supply == g_supply,
+                GEra.start_date <= start_date,
+                or_(GEra.finish_date == null(), GEra.finish_date >= start_date),
+            )
+        ).one_or_none()
+        if existing_g_era is None:
+            raise BadRequest("The start date is outside the dates of the supply.")
+
+        site_code = add_arg(args, "Site Code", vals, 2)
+        physical_site = None
+        logical_sites = []
+        if site_code == NO_CHANGE:
+            for site_g_era in existing_g_era.site_g_eras:
+                if site_g_era.is_physical:
+                    physical_site = site_g_era.site
+                else:
+                    logical_sites.append(site_g_era.site)
+        else:
+            physical_site = Site.get_by_code(sess, site_code)
+
+        msn = add_arg(args, "Meter Serial Number", vals, 3)
+        if msn == NO_CHANGE:
+            msn = existing_g_era.msn
+
+        correction_factor_str = add_arg(args, "Correction Factor", vals, 4)
+        if correction_factor_str == NO_CHANGE:
+            correction_factor = existing_g_era.correction_factor
+        else:
+            correction_factor = Decimal(correction_factor_str)
+
+        g_unit_code = add_arg(args, "Unit", vals, 5)
+        if g_unit_code == NO_CHANGE:
+            g_unit = existing_g_era.g_unit
+        else:
+            g_unit = GUnit.get_by_code(sess, g_unit_code)
+
+        contract_name = add_arg(args, "Supplier Contract Name", vals, 6)
+        if contract_name == NO_CHANGE:
+            g_contract = existing_g_era.g_contract
+        else:
+            g_contract = GContract.get_supplier_by_name(sess, contract_name)
+
+        account = add_arg(args, "Account", vals, 7)
+        if account == NO_CHANGE:
+            account = existing_g_era.account
+
+        g_reading_frequency_code = add_arg(args, "Reading Frequency", vals, 8)
+        if g_reading_frequency_code == NO_CHANGE:
+            g_reading_frequency = existing_g_era.g_reading_frequency
+        else:
+            g_reading_frequency = GReadingFrequency.get_by_code(
+                sess, g_reading_frequency_code
+            )
+
+        aq_str = add_arg(args, "AQ", vals, 9)
+        aq = existing_g_era.aq if aq_str == NO_CHANGE else Decimal(aq_str)
+
+        soq_str = add_arg(args, "SOQ", vals, 10)
+        soq = existing_g_era.soq if soq_str == NO_CHANGE else Decimal(soq_str)
+
+        g_supply.insert_g_era(
+            sess,
+            physical_site,
+            logical_sites,
+            start_date,
+            None,
+            msn,
+            correction_factor,
+            g_unit,
+            g_contract,
+            account,
+            g_reading_frequency,
+            aq,
+            soq,
+        )
+        sess.flush()
+
+    elif action == "update":
+        mprn = add_arg(args, "MPRN", vals, 0)
+        g_supply = GSupply.get_by_mprn(sess, mprn)
+        date_str = add_arg(args, "date", vals, 1)
+        dt = parse_hh_start(date_str)
+        g_era = g_supply.find_g_era_at(sess, dt)
+        if g_era is None:
+            raise BadRequest("There isn't a gas era at this date.")
+
+        start_date_str = add_arg(args, "Start Date", vals, 2)
+        if start_date_str == NO_CHANGE:
+            start_date = g_era.start_date
+        else:
+            start_date = parse_hh_start(start_date_str)
+
+        finish_date_str = add_arg(args, "Finish Date", vals, 3)
+        if finish_date_str == NO_CHANGE:
+            finish_date = g_era.finish_date
+        else:
+            finish_date = parse_hh_start(finish_date_str)
+
+        msn = add_arg(args, "Meter Serial Number", vals, 4)
+        if msn == NO_CHANGE:
+            msn = g_era.msn
+
+        correction_factor_str = add_arg(args, "Correction Factor", vals, 5)
+        if correction_factor_str == NO_CHANGE:
+            correction_factor = g_era.correction_factor
+        else:
+            correction_factor = Decimal(correction_factor_str)
+
+        g_unit_code = add_arg(args, "Unit", vals, 6)
+        if g_unit_code == NO_CHANGE:
+            g_unit = g_era.g_unit
+        else:
+            g_unit = GUnit.get_by_code(sess, g_unit_code)
+
+        contract_name = add_arg(args, "Supplier Contract Name", vals, 7)
+        if contract_name == NO_CHANGE:
+            g_contract = g_era.g_contract
+        else:
+            g_contract = GContract.get_supplier_by_name(sess, contract_name)
+
+        account = add_arg(args, "Account", vals, 8)
+        if account == NO_CHANGE:
+            account = g_era.account
+
+        g_reading_frequency_code = add_arg(args, "Reading Frequency", vals, 9)
+        if g_reading_frequency_code == NO_CHANGE:
+            g_reading_frequency = g_era.g_reading_frequency
+        else:
+            g_reading_frequency = GReadingFrequency.get_by_code(
+                sess, g_reading_frequency_code
+            )
+
+        aq_str = add_arg(args, "AQ", vals, 10)
+        aq = g_era.aq if aq_str == NO_CHANGE else Decimal(aq_str)
+
+        soq_str = add_arg(args, "SOQ", vals, 11)
+        soq = g_era.aq if soq_str == NO_CHANGE else Decimal(soq_str)
+
+        g_supply.update_g_era(
+            sess,
+            g_era,
+            start_date,
+            finish_date,
+            msn,
+            correction_factor,
+            g_unit,
+            g_contract,
+            account,
+            g_reading_frequency,
+            aq,
+            soq,
+        )
+    elif action == "delete":
+        mprn = add_arg(args, "MPRN", vals, 0)
+        g_supply = GSupply.get_by_mprn(sess, mprn)
+        date_str = add_arg(args, "Date", vals, 1)
+        dt = parse_hh_start(date_str)
+        g_era = g_supply.find_g_era_at(sess, dt)
+        if g_era is None:
+            raise BadRequest("There isn't a gas era at this date.")
+
+        g_supply.delete_g_era(sess, g_era)
+        sess.flush()
+
+
 def general_import_g_batch(sess, action, vals, args):
     if action == "insert":
         contract_name = add_arg(args, "Supplier Contract Name", vals, 0)
@@ -751,47 +942,70 @@ def general_import_bill(sess, action, vals, args):
             supply,
         )
 
-        for i in range(15, len(vals), 11):
-            msn = add_arg(args, "Meter Serial Number", vals, i)
-            mpan_str = add_arg(args, "MPAN", vals, i + 1)
-            coefficient_str = add_arg(args, "Coefficient", vals, i + 2)
-            coefficient = Decimal(coefficient_str)
-            units = add_arg(args, "Units", vals, i + 3)
-            tpr_code = add_arg(args, "TPR", vals, i + 4)
-            if len(tpr_code) > 0:
-                tpr = Tpr.get_by_code(sess, tpr_code)
-            else:
-                tpr = None
+        i = 15
+        while i < len(vals):
+            typ = add_arg(args, "read or element", vals, i)
+            if typ == "read":
+                msn = add_arg(args, "Meter Serial Number", vals, i + 1)
+                mpan_str = add_arg(args, "MPAN", vals, i + 2)
+                coefficient_str = add_arg(args, "Coefficient", vals, i + 3)
+                coefficient = Decimal(coefficient_str)
+                units = add_arg(args, "Units", vals, i + 4)
+                tpr_code = add_arg(args, "TPR", vals, i + 5)
+                if len(tpr_code) > 0:
+                    tpr = Tpr.get_by_code(sess, tpr_code)
+                else:
+                    tpr = None
 
-            prev_date_str = add_arg(args, "Previous Date", vals, i + 5)
-            prev_date = parse_hh_start(prev_date_str)
-            prev_value_str = add_arg(args, "Previous Value", vals, i + 6)
-            prev_value = Decimal(prev_value_str)
+                prev_date_str = add_arg(args, "Previous Date", vals, i + 6)
+                prev_date = parse_hh_start(prev_date_str)
+                prev_value_str = add_arg(args, "Previous Value", vals, i + 7)
+                prev_value = Decimal(prev_value_str)
 
-            prev_type_str = add_arg(args, "Previous Type", vals, i + 7)
-            prev_type = ReadType.get_by_code(sess, prev_type_str)
+                prev_type_str = add_arg(args, "Previous Type", vals, i + 8)
+                prev_type = ReadType.get_by_code(sess, prev_type_str)
 
-            pres_date_str = add_arg(args, "Present Date", vals, i + 8)
-            pres_date = parse_hh_start(pres_date_str)
-            pres_value_str = add_arg(args, "Present Value", vals, i + 9)
-            pres_value = Decimal(pres_value_str)
+                pres_date_str = add_arg(args, "Present Date", vals, i + 9)
+                pres_date = parse_hh_start(pres_date_str)
+                pres_value_str = add_arg(args, "Present Value", vals, i + 10)
+                pres_value = Decimal(pres_value_str)
 
-            pres_type_str = add_arg(args, "Present Type", vals, i + 10)
-            pres_type = ReadType.get_by_code(sess, pres_type_str)
-            bill.insert_read(
-                sess,
-                tpr,
-                coefficient,
-                units,
-                msn,
-                mpan_str,
-                prev_date,
-                prev_value,
-                prev_type,
-                pres_date,
-                pres_value,
-                pres_type,
-            )
+                pres_type_str = add_arg(args, "Present Type", vals, i + 11)
+                pres_type = ReadType.get_by_code(sess, pres_type_str)
+                bill.insert_read(
+                    sess,
+                    tpr,
+                    coefficient,
+                    units,
+                    msn,
+                    mpan_str,
+                    prev_date,
+                    prev_value,
+                    prev_type,
+                    pres_date,
+                    pres_value,
+                    pres_type,
+                )
+                i += 12
+            elif typ == "element":
+                name = add_arg(args, "Name", vals, i + 1)
+                start_date_str = add_arg(args, "Start Date", vals, i + 2)
+                start_date = parse_hh_start(start_date_str)
+                finish_date_str = add_arg(args, "Finish Date", vals, i + 3)
+                finish_date = parse_hh_start(finish_date_str)
+                net_str = add_arg(args, "Net", vals, i + 4)
+                net = Decimal(net_str)
+                breakdown_str = add_arg(args, "Breakdown", vals, i + 5)
+                breakdown = _parse_breakdown(breakdown_str)
+                bill.insert_element(
+                    sess,
+                    name,
+                    start_date,
+                    finish_date,
+                    net,
+                    breakdown,
+                )
+                i += 6
 
     elif action == "update":
         bill_id_str = add_arg(args, "Bill Id", vals, 0)
@@ -862,6 +1076,12 @@ def general_import_bill(sess, action, vals, args):
             breakdown,
         )
 
+    elif action == "delete":
+        bill_id_str = add_arg(args, "Bill Chellow Id", vals, 0)
+        bill_id = int(bill_id_str)
+        bill = Bill.get_by_id(sess, bill_id)
+        bill.delete(sess)
+
 
 def general_import_g_bill(sess, action, vals, args):
     if action == "insert":
@@ -955,6 +1175,77 @@ def general_import_g_bill(sess, action, vals, args):
             )
 
 
+def general_import_g_register_read(sess, action, vals, args):
+    if action == "insert":
+        contract_name = add_arg(args, "Supplier Contract Name", vals, 0)
+
+        g_contract = GContract.get_supplier_by_name(sess, contract_name)
+
+        batch_reference = add_arg(args, "Batch Reference", vals, 1)
+
+        g_batch = g_contract.get_g_batch_by_reference(sess, batch_reference)
+
+        mprn = add_arg(args, "MPRN", vals, 2)
+        g_supply = GSupply.get_by_mprn(sess, mprn)
+
+        bill_start_date_str = add_arg(args, "Bill Start Date", vals, 3)
+        bill_start_date = parse_hh_start(bill_start_date_str)
+
+        g_bill = sess.scalars(
+            select(GBill).where(
+                GBill.g_batch == g_batch,
+                GBill.g_supply == g_supply,
+                GBill.start_date == bill_start_date,
+            )
+        ).first()
+
+        if g_bill is None:
+            raise BadRequest(
+                f"Can't find a bill in batch {batch_reference} in contract "
+                f"{contract_name} with MPRN {mprn} starting at "
+                f"{hh_format(bill_start_date)}"
+            )
+
+        for i in range(4, len(vals), 10):
+            msn = add_arg(args, "Meter Serial Number", vals, i)
+            g_unit_code = add_arg(args, "Unit", vals, i + 1)
+            g_unit = GUnit.get_by_code(sess, g_unit_code)
+            correction_factor_str = add_arg(args, "Correction Factor", vals, i + 2)
+            correction_factor = Decimal(correction_factor_str)
+            calorific_value_str = add_arg(args, "Calorific Value", vals, i + 3)
+            calorific_value = Decimal(calorific_value_str)
+
+            prev_date_str = add_arg(args, "Previous Date", vals, i + 4)
+            prev_date = parse_hh_start(prev_date_str)
+            prev_value_str = add_arg(args, "Previous Value", vals, i + 5)
+            prev_value = Decimal(prev_value_str)
+
+            prev_type_str = add_arg(args, "Previous Type", vals, i + 6)
+            prev_type = GReadType.get_by_code(sess, prev_type_str)
+
+            pres_date_str = add_arg(args, "Present Date", vals, i + 7)
+            pres_date = parse_hh_start(pres_date_str)
+            pres_value_str = add_arg(args, "Present Value", vals, i + 8)
+            pres_value = Decimal(pres_value_str)
+
+            pres_type_str = add_arg(args, "Present Type", vals, i + 9)
+            pres_type = GReadType.get_by_code(sess, pres_type_str)
+
+            g_bill.insert_g_read(
+                sess,
+                msn,
+                g_unit,
+                correction_factor,
+                calorific_value,
+                prev_value,
+                prev_date,
+                prev_type,
+                pres_value,
+                pres_date,
+                pres_type,
+            )
+
+
 def general_import_register_read(sess, action, vals, args):
     if action == "insert":
         pass
@@ -1069,20 +1360,27 @@ def general_import_supply(sess, action, vals, args):
         dc_account = add_arg(args, "DC Account", vals, 10)
         msn = add_arg(args, "Meter Serial Number", vals, 11)
         dno_code = add_arg(args, "DNO Code", vals, 12)
-        dno = Party.get_dno_by_code(sess, dno_code)
+        dno = Party.get_dno_by_code(sess, dno_code, start_date)
         pc_code = add_arg(args, "Profile Class", vals, 13)
         pc = Pc.get_by_code(sess, parse_pc_code(pc_code))
         mtc_code = add_arg(args, "Meter Timeswitch Class", vals, 14)
         cop_code = add_arg(args, "CoP", vals, 15)
         cop = Cop.get_by_code(sess, cop_code)
-        ssc_code = add_arg(args, "Standard Settlement Configuration", vals, 16)
-        ssc = Ssc.get_by_code(sess, ssc_code) if len(ssc_code) > 0 else None
-        properties_str = add_arg(args, "Properties", vals, 17)
-        try:
-            properties = loads(properties_str)
-        except ZishException as e:
-            raise BadRequest("Can't parse the properties field. " + str(e))
-        imp_mpan_core = add_arg(args, "Import MPAN Core", vals, 18)
+        comm_code = add_arg(args, "Comms Type", vals, 16)
+        comm = Comm.get_by_code(sess, comm_code)
+        ssc_code_str = add_arg(args, "Standard Settlement Configuration", vals, 17)
+        if len(ssc_code_str) > 0:
+            ssc = Ssc.get_by_code(sess, ssc_code_str, start_date)
+            ssc_code = ssc.code
+        else:
+            ssc_code = None
+        energisation_status_code = add_arg(args, "Energisation Status", vals, 18)
+        energisation_status = EnergisationStatus.get_by_code(
+            sess, energisation_status_code
+        )
+        dtc_meter_type_code = add_arg(args, "DTC Meter Type", vals, 19)
+        dtc_meter_type = DtcMeterType.get_by_code(sess, dtc_meter_type_code)
+        imp_mpan_core = add_arg(args, "Import MPAN Core", vals, 20)
         if len(imp_mpan_core) == 0:
             imp_mpan_core = None
         else:
@@ -1094,55 +1392,48 @@ def general_import_supply(sess, action, vals, args):
             imp_supplier_account = None
             imp_sc = None
         else:
-            imp_llfc_code = add_arg(args, "Import LLFC", vals, 19)
-            imp_sc_str = add_arg(args, "Import Agreed Supply Capacity", vals, 20)
+            imp_llfc_code = add_arg(args, "Import LLFC", vals, 21)
+            imp_sc_str = add_arg(args, "Import Agreed Supply Capacity", vals, 22)
             try:
                 imp_sc = int(imp_sc_str)
             except ValueError as e:
-                raise BadRequest(
-                    "The import supply capacity must be an integer." + str(e)
-                )
+                raise BadRequest(f"The import supply capacity must be an integer. {e}")
 
             imp_supplier_contract_name = add_arg(
-                args, "Import Supplier Contract", vals, 21
+                args, "Import Supplier Contract", vals, 23
             )
-            imp_supplier_account = add_arg(args, "Import Supplier Account", vals, 22)
+            imp_supplier_account = add_arg(args, "Import Supplier Account", vals, 24)
             imp_supplier_contract = Contract.get_supplier_by_name(
                 sess, imp_supplier_contract_name
             )
 
-        exp_supplier_contract = None
-        exp_sc = None
-        exp_llfc_code = None
-        exp_mpan_core = None
+        exp_supplier_contract = exp_sc = exp_llfc_code = exp_mpan_core = None
         exp_supplier_account = None
-        if len(vals) > 22:
-            exp_mpan_core = add_arg(args, "Export MPAN Core", vals, 23)
+        if len(vals) > 25:
+            exp_mpan_core = add_arg(args, "Export MPAN Core", vals, 25)
             if len(exp_mpan_core) == 0:
                 exp_mpan_core = None
             else:
                 exp_mpan_core = parse_mpan_core(exp_mpan_core)
 
             if exp_mpan_core is not None:
-                exp_llfc_code = add_arg(args, "Export LLFC", vals, 24)
-                exp_sc_str = add_arg(args, "Export Agreed Supply Capacity", vals, 25)
+                exp_llfc_code = add_arg(args, "Export LLFC", vals, 26)
+                exp_sc_str = add_arg(args, "Export Agreed Supply Capacity", vals, 27)
                 try:
                     exp_sc = int(exp_sc_str)
                 except ValueError as e:
                     raise BadRequest(
-                        "The export agreed supply capacity "
-                        + "must be an integer."
-                        + str(e)
+                        f"The export agreed supply capacity must be an integer. {e}"
                     )
 
                 exp_supplier_contract_name = add_arg(
-                    args, "Export Supplier Contract", vals, 26
+                    args, "Export Supplier Contract", vals, 28
                 )
                 exp_supplier_contract = Contract.get_supplier_by_name(
                     sess, exp_supplier_contract_name
                 )
                 exp_supplier_account = add_arg(
-                    args, "Export Supplier Account", vals, 27
+                    args, "Export Supplier Account", vals, 29
                 )
 
         supply = site.insert_e_supply(
@@ -1162,8 +1453,10 @@ def general_import_supply(sess, action, vals, args):
             pc,
             mtc_code,
             cop,
-            ssc,
-            properties,
+            comm,
+            ssc_code,
+            energisation_status,
+            dtc_meter_type,
             imp_mpan_core,
             imp_llfc_code,
             imp_supplier_contract,
@@ -1385,7 +1678,11 @@ def general_import_site(sess, action, vals, args):
             site.delete()
         elif action == "update":
             new_code = add_arg(args, "new_site_code", vals, 1)
+            if new_code == NO_CHANGE:
+                new_code = site.code
             name = add_arg(args, "new_site_name", vals, 2)
+            if name == NO_CHANGE:
+                name = site.name
             site.update(new_code, name)
 
 
@@ -1508,21 +1805,43 @@ for k in tuple(globals().keys()):
         typ_funcs[k[len(PREFIX) :]] = globals()[k]
 
 
+def _process_line(sess, hh_data, line, args):
+
+    action = add_arg(args, "action", line, 0).lower()
+    if action not in ALLOWED_ACTIONS:
+        raise BadRequest(f"The 'action' field must be one of {ALLOWED_ACTIONS}")
+    typ = add_arg(args, "type", line, 1).lower()
+    vals = line[2:]
+    if typ == "hh_datum":
+        if action == "insert":
+            hh_data.append(
+                {
+                    "mpan_core": parse_mpan_core(add_arg(args, "MPAN Core", vals, 0)),
+                    "start_date": parse_hh_start(add_arg(args, "Start Date", vals, 1)),
+                    "channel_type": parse_channel_type(
+                        add_arg(args, "Channel Type", vals, 2)
+                    ),
+                    "value": Decimal(add_arg(args, "Value", vals, 3)),
+                    "status": add_arg(args, "Status", vals, 4),
+                }
+            )
+    else:
+        try:
+            typ_func = typ_funcs[typ]
+            typ_func(sess, action, vals, args)
+        except KeyError:
+            raise BadRequest(f"The type {typ} is not recognized.")
+
+
 class GeneralImporter(threading.Thread):
     def __init__(self, f):
         threading.Thread.__init__(self)
         self.line_number = None
         self.f = f
-        self.rd_lock = threading.Lock()
         self.error_message = None
         self.args = []
-        self.hh_data = []
-
-    def get_fields(self):
-        fields = {"line_number": self.line_number, "error_message": self.error_message}
-        if self.error_message is not None:
-            fields["csv_line"] = self.args
-        return fields
+        self.failed_args = []
+        self.failed_lines = []
 
     def run(self):
         try:
@@ -1539,63 +1858,32 @@ class GeneralImporter(threading.Thread):
                     if len(line) > 0 and line[0].startswith("#"):
                         continue
 
-                    action = add_arg(self.args, "action", line, 0).lower()
-                    if action not in ALLOWED_ACTIONS:
-                        raise BadRequest(
-                            f"The 'action' field must be one of {ALLOWED_ACTIONS}"
-                        )
-                    typ = add_arg(self.args, "type", line, 1).lower()
-                    vals = line[2:]
-                    if typ == "hh_datum":
-                        if action == "insert":
-                            hh_data.append(
-                                {
-                                    "mpan_core": parse_mpan_core(
-                                        add_arg(self.args, "MPAN Core", vals, 0)
-                                    ),
-                                    "start_date": parse_hh_start(
-                                        add_arg(self.args, "Start Date", vals, 1)
-                                    ),
-                                    "channel_type": parse_channel_type(
-                                        add_arg(self.args, "Channel Type", vals, 2)
-                                    ),
-                                    "value": Decimal(
-                                        add_arg(self.args, "Value", vals, 3)
-                                    ),
-                                    "status": add_arg(self.args, "Status", vals, 4),
-                                }
-                            )
-                    else:
-                        try:
-                            typ_func = typ_funcs[typ]
-                            typ_func(sess, action, vals, self.args)
-                        except KeyError:
-                            raise BadRequest(f"The type {typ} is not recognized.")
+                    try:
+                        with sess.begin_nested():
+                            _process_line(sess, hh_data, line, self.args)
+
+                    except BadRequest as e:
+                        self.args.insert(0, ("Error", e.description))
+                        self.failed_args.append(self.args)
+                        self.failed_lines.append(line)
 
                 HhDatum.insert(sess, hh_data)
-                sess.commit()
+
+                if len(self.failed_lines) == 0:
+                    sess.commit()
+                else:
+                    sess.rollback()
         except BadRequest as e:
-            try:
-                self.rd_lock.acquire()
-                self.error_message = e.description
-            finally:
-                self.rd_lock.release()
+            self.error_message = e.description
         except BaseException:
-            try:
-                self.rd_lock.acquire()
-                self.error_message = traceback.format_exc()
-            finally:
-                self.rd_lock.release()
+            self.error_message = traceback.format_exc()
 
 
 def start_process(f):
-    try:
+    with process_lock:
         global process_id
-        process_lock.acquire()
         proc_id = process_id
         process_id += 1
-    finally:
-        process_lock.release()
 
     process = GeneralImporter(f)
     processes[proc_id] = process
@@ -1604,16 +1892,9 @@ def start_process(f):
 
 
 def get_process_ids():
-    try:
-        process_lock.acquire()
+    with process_lock:
         return processes.keys()
-    finally:
-        process_lock.release()
 
 
 def get_process(id):
-    try:
-        process_lock.acquire()
-        return processes[id]
-    finally:
-        process_lock.release()
+    return processes[id]
