@@ -20,8 +20,8 @@ importer = None
 
 
 def api_get(s, path, params=None):
-    url = f"https://api.nationalgrideso.com/api/3/action/{path}"
-    res = s.get(url, params=params)
+    url = f"https://api.neso.energy/api/3/action/{path}"
+    res = s.get(url, params=params, timeout=120)
     try:
         res_j = res.json()
     except requests.exceptions.JSONDecodeError as e:
@@ -41,7 +41,7 @@ def run_import(sess, log, set_progress):
     s = requests.Session()
     s.verify = False
 
-    for mod_name in ("chellow.e.tnuos", "chellow.e.bsuos"):
+    for mod_name in ("chellow.e.tnuos", "chellow.e.bsuos", "chellow.e.triad"):
         mod = import_module(mod_name)
         mod.national_grid_import(sess, log, set_progress, s)
 
