@@ -72,7 +72,7 @@ def run_import(sess, log, set_progress):
     paths_list = []
     for sub_entry in tree_entry["tree"]:
         path = sub_entry["path"].split("/")
-        if path[-1] == "README.md":
+        if path[-1].upper() == "README.MD":
             continue
         if len(path) == 1 and path[0] == "LICENSE":
             continue
@@ -83,11 +83,13 @@ def run_import(sess, log, set_progress):
     paths = tuple(paths_list)
 
     for mod_name in (
-        "chellow.e.bsuos",
-        "chellow.e.dno_rate_parser",
-        "chellow.e.laf_import",
         "chellow.e.mdd_importer",
-        "chellow.e.tnuos",
+        "chellow.e.bsuos",
+        "chellow.e.ccl",
+        "chellow.e.dno_rate_parser",
+        "chellow.e.triad",
+        "chellow.e.lafs",
+        "chellow.gas.ccl",
         "chellow.gas.dn_rate_parser",
     ):
         mod = import_module(mod_name)
