@@ -1,20 +1,18 @@
 import csv
-import os
 import sys
 import threading
 import traceback
 from datetime import datetime as Datetime
 
-from flask import g
+from flask import g, redirect
 
 from sqlalchemy import or_, text
 from sqlalchemy.sql.expression import null
 
-import chellow.dloads
+from chellow.dloads import open_file
 from chellow.e.computer import SiteSource, contract_func, displaced_era, forecast_date
 from chellow.models import Contract, Era, Session, Site, SiteEra, Source, Supply
 from chellow.utils import c_months_u, hh_format, hh_range, req_int
-from chellow.views import chellow_redirect
 
 
 def to_val(v):
@@ -34,10 +32,7 @@ def content(contract_id, end_year, end_month, months, user):
     f = None
     try:
         with Session() as sess:
-            running_name, finished_name = chellow.dloads.make_names(
-                "displaced.csv", user
-            )
-            f = open(running_name, mode="w", newline="")
+            f = open_file("displaced.csv", user, mode="w", newline="")
             writer = csv.writer(f, lineterminator="\n")
             titles = [
                 "Site Code",
@@ -70,7 +65,8 @@ def content(contract_id, end_year, end_month, months, user):
                     or_(Era.finish_date == null(), Era.finish_date >= start_date),
                     Era.start_date <= finish_date,
                     or_(
-                        Source.code.in_(("gen", "gen-net")), Era.exp_mpan_core != null()
+                        Source.code.in_(("gen", "gen-grid")),
+                        Era.exp_mpan_core != null(),
                     ),
                 )
                 .distinct()
@@ -157,7 +153,7 @@ def content(contract_id, end_year, end_month, months, user):
                                 "channel.era_id = era.id and era.supply_id = "
                                 "supply.id and supply.source_id = source.id and "
                                 "channel.channel_type = 'ACTIVE' and not "
-                                "(source.code = 'net' and channel.imp_related "
+                                "(source.code = 'grid' and channel.imp_related "
                                 "is true) and hh_datum.start_date >= "
                                 ":month_start and hh_datum.start_date "
                                 "<= :month_finish and "
@@ -185,17 +181,17 @@ def content(contract_id, end_year, end_month, months, user):
                         gen_breakdown = {}
                         exported = 0
                         while hh_start == hh_date:
-                            if not imp_related and source_code in ("net", "gen-net"):
+                            if not imp_related and source_code in ("grid", "gen-grid"):
                                 exported += hh_val
                             if (imp_related and source_code == "gen") or (
-                                not imp_related and source_code == "gen-net"
+                                not imp_related and source_code == "gen-grid"
                             ):
                                 gen_breakdown[gen_type_code] = (
                                     gen_breakdown.setdefault(gen_type_code, 0) + hh_val
                                 )
 
                             if (not imp_related and source_code == "gen") or (
-                                imp_related and source_code == "gen-net"
+                                imp_related and source_code == "gen-grid"
                             ):
                                 gen_breakdown[gen_type_code] = (
                                     gen_breakdown.setdefault(gen_type_code, 0) - hh_val
@@ -268,7 +264,6 @@ def content(contract_id, end_year, end_month, months, user):
     finally:
         if f is not None:
             f.close()
-            os.rename(running_name, finished_name)
 
 
 def do_get(sess):
@@ -278,4 +273,4 @@ def do_get(sess):
     contract_id = req_int("supplier_contract_id")
     args = contract_id, end_year, end_month, months, g.user
     threading.Thread(target=content, args=args).start()
-    return chellow_redirect("/downloads", 303)
+    return redirect("/downloads", 303)
