@@ -688,8 +688,13 @@ def _process_period(
 
     for elname, val_elem in val_elems.items():
         for part_name, part in val_elem["parts"].items():
-            virtual_part = part.get("virtual")
-            actual_part = part.get("actual")
+            if part_name == "gbp":
+                virtual_part = round(part.get("virtual", 0), 2)
+                actual_part = part.get("actual", 0)
+            else:
+                virtual_part = part.get("virtual")
+                actual_part = part.get("actual")
+
             if isinstance(virtual_part, set) and len(virtual_part) == 1:
                 virtual_part = next(iter(virtual_part))
             if isinstance(actual_part, set) and len(actual_part) == 1:
