@@ -1,16 +1,14 @@
-import os
 import threading
 import traceback
-import zipfile
 from datetime import datetime as Datetime
 
-from flask import g, request
+from flask import g, redirect, request
 
 from sqlalchemy import null, or_, select, text, true
 
 from werkzeug.exceptions import BadRequest
 
-import chellow.dloads
+from chellow.dloads import open_file
 from chellow.models import Era, Session, Site, SiteEra, Supply, User
 from chellow.utils import (
     hh_format,
@@ -22,7 +20,6 @@ from chellow.utils import (
     req_str,
     to_ct,
 )
-from chellow.views import chellow_redirect
 
 
 def csv_str(row):
@@ -67,12 +64,10 @@ def content(start_date, finish_date, supply_id, mpan_cores, is_zipped, user_id):
         with Session() as sess:
             user = User.get_by_id(sess, user_id)
 
-            running_name, finished_name = chellow.dloads.make_names(base_name, user)
-
             if is_zipped:
-                zf = zipfile.ZipFile(running_name, "w")
+                zf = open_file(base_name, user, mode="w", is_zip=True)
             else:
-                tmp_file = open(running_name, "w")
+                tmp_file = open_file(base_name, user, mode="w")
 
             caches = {}
             supplies = (
@@ -243,11 +238,9 @@ def content(start_date, finish_date, supply_id, mpan_cores, is_zipped, user_id):
     finally:
         if is_zipped:
             zf.close()
-            os.rename(running_name, finished_name)
         else:
             if tmp_file is not None:
                 tmp_file.close()
-                os.rename(running_name, finished_name)
 
 
 def do_post(sess):
@@ -273,4 +266,4 @@ def do_post(sess):
     args = start_date, finish_date, supply_id, mpan_cores, is_zipped, g.user.id
 
     threading.Thread(target=content, args=args).start()
-    return chellow_redirect("/downloads", 303)
+    return redirect("/downloads", 303)
