@@ -1,23 +1,22 @@
 import csv
-import os
 import threading
 import traceback
 from collections import defaultdict
 
-from flask import g
+from flask import g, redirect
 
 from sqlalchemy import select
 from sqlalchemy.orm import joinedload
 
-import chellow.dloads
+from chellow.dloads import open_file
 from chellow.models import Batch, Bill, Session, User
 from chellow.utils import csv_make_val, req_int
-from chellow.views import chellow_redirect
 
 
 def _content(sess, writer, batch_id):
     batch = Batch.get_by_id(sess, batch_id)
     titles = [
+        "chellow_id",
         "supplier_contract",
         "batch_reference",
         "bill_reference",
@@ -37,6 +36,7 @@ def _content(sess, writer, batch_id):
         "vat_2_percent",
         "vat_2_net",
         "vat_2_vat",
+        "breakdown",
     ]
     writer.writerow(titles)
 
@@ -61,6 +61,7 @@ def _content(sess, writer, batch_id):
                 vbd["net"] += vat_vals["net"]
 
         vals = {
+            "chellow_id": bill.id,
             "supplier_contract": batch.contract.name,
             "batch_reference": batch.reference,
             "bill_reference": bill.reference,
@@ -80,8 +81,9 @@ def _content(sess, writer, batch_id):
             "vat_2_percent": None,
             "vat_2_net": None,
             "vat_2_vat": None,
+            "breakdown": bill.breakdown,
         }
-        for i, (percentage, vbd) in enumerate(sorted(vat_breakdown), 1):
+        for i, (percentage, vbd) in enumerate(sorted(vat_breakdown.items()), 1):
             vals[f"vat_{i}_percentage"] = percentage
             vals[f"vat_{i}_net"] = vbd["net"]
             vals[f"vat_{i}_vat"] = vbd["vat"]
@@ -94,10 +96,7 @@ def content(user_id, batch_id):
     try:
         with Session() as sess:
             user = User.get_by_id(sess, user_id)
-            running_name, finished_name = chellow.dloads.make_names(
-                f"bills_batch_{batch_id}.csv", user
-            )
-            f = open(running_name, mode="w", newline="")
+            f = open_file(f"bills_batch_{batch_id}.csv", user, mode="w", newline="")
             writer = csv.writer(f, lineterminator="\n")
             _content(sess, writer, batch_id)
 
@@ -109,11 +108,10 @@ def content(user_id, batch_id):
     finally:
         if f is not None:
             f.close()
-            os.rename(running_name, finished_name)
 
 
 def do_get(sess):
     batch_id = req_int("batch_id")
     args = g.user.id, batch_id
     threading.Thread(target=content, args=args).start()
-    return chellow_redirect("/downloads", 303)
+    return redirect("/downloads", 303)
