@@ -5,6 +5,7 @@ from io import BytesIO
 
 from chellow.e.bill_parsers.haven_edi_tprs import (
     Parser,
+    _process_BTL,
     _process_CCD1,
     _process_CCD3,
     _process_MAN,
@@ -12,10 +13,51 @@ from chellow.e.bill_parsers.haven_edi_tprs import (
     _process_MTR,
     _process_VAT,
 )
-from chellow.utils import utc_datetime
+from chellow.utils import ct_datetime, to_utc, utc_datetime
 
 
-def test_process_MTR_UTLHDR(mocker):
+def test_process_BTL():
+    elements = {
+        "UVLT": ["7630", "R"],
+        "UTVA": ["1361", "R"],
+        "TBTL": ["8293", "R"],
+    }
+    issue_date = to_utc(ct_datetime(2025, 8, 3))
+    start_date = to_utc(ct_datetime(2025, 7, 1))
+    finish_date = to_utc(ct_datetime(2025, 7, 31, 23, 30))
+    headers = {
+        "bill_type_code": "N",
+        "account": "hgteiulh",
+        "mpan_core": "22 8872 1749 119",
+        "reference": "dskghksldfj",
+        "issue_date": issue_date,
+        "start_date": start_date,
+        "finish_date": finish_date,
+        "kwh": Decimal("77.34"),
+        "breakdown": {},
+        "reads": [],
+        "elements": [],
+    }
+    bill = _process_BTL(elements, headers)
+    assert bill == {
+        "account": "hgteiulh",
+        "bill_type_code": "N",
+        "breakdown": {},
+        "elements": [],
+        "finish_date": finish_date,
+        "gross": Decimal("-82.93"),
+        "issue_date": issue_date,
+        "kwh": Decimal("77.34"),
+        "mpan_core": "22 8872 1749 119",
+        "net": Decimal("-76.30"),
+        "reads": [],
+        "reference": "dskghksldfj",
+        "start_date": start_date,
+        "vat": Decimal("-13.61"),
+    }
+
+
+def test_process_MTR_UTLHDR():
     elements = {}
     headers = {"message_type": "UTLHDR", "breakdown": {}}
     bill = _process_MTR(elements, headers)
@@ -245,6 +287,7 @@ def test_process_MHD(mocker):
         "is_ebatch": False,
         "issue_date": None,
         "bill_type_code": None,
+        "elements": [],
     }
     assert headers == expected_headers
     assert isinstance(headers["breakdown"], type(expected_headers["breakdown"]))
@@ -257,15 +300,25 @@ def test_process_CCD3(mocker):
         "TMOD": ["453043"],
         "CONS": [[]],
         "BPRI": ["10"],
+        "CSDT": ["250401"],
+        "CEDT": ["250430"],
     }
 
-    headers = {"kwh": Decimal("0"), "breakdown": {}}
+    headers = {"kwh": Decimal("0"), "elements": []}
 
     _process_CCD3(elements, headers)
 
     expected_headers = {
         "kwh": Decimal("0"),
-        "breakdown": {},
+        "elements": [
+            {
+                "name": "453043",
+                "start_date": to_utc(ct_datetime(2025, 4, 1, 0, 0)),
+                "finish_date": to_utc(ct_datetime(2025, 4, 30, 23, 30)),
+                "net": Decimal("0.00"),
+                "breakdown": {},
+            }
+        ],
     }
     assert headers == expected_headers
 
@@ -276,17 +329,28 @@ def test_process_CCD3_ebrs_kwh(mocker):
         "TCOD": ["R10001", "Day"],
         "TMOD": ["823408"],
         "NUCT": ["4127137"],
+        "CSDT": ["250401"],
+        "CEDT": ["250430"],
     }
 
-    headers = {"kwh": Decimal("0"), "breakdown": defaultdict(int, {})}
+    headers = {"elements": [], "kwh": Decimal("0"), "breakdown": {}}
 
     _process_CCD3(elements, headers)
 
     expected_headers = {
         "kwh": Decimal("0"),
-        "breakdown": {
-            "ebrs-kwh": Decimal("4127.137"),
-        },
+        "breakdown": {},
+        "elements": [
+            {
+                "name": "ebrs",
+                "start_date": to_utc(ct_datetime(2025, 4, 1, 0, 0)),
+                "finish_date": to_utc(ct_datetime(2025, 4, 30, 23, 30)),
+                "net": Decimal("0.00"),
+                "breakdown": {
+                    "kwh": Decimal("4127.137"),
+                },
+            }
+        ],
     }
     assert headers == expected_headers
 
@@ -310,12 +374,13 @@ def test_process_CCD_1(mocker):
 
     headers = {
         "reads": [],
+        "elements": [],
     }
 
     _process_CCD1(elements, headers)
 
     expected_headers = {
-        "mpan_core": "22 7583 4732 592",
+        "elements": [],
         "reads": [
             {
                 "msn": msn,
@@ -350,7 +415,8 @@ def test_process_VAT(mocker):
         "vat": Decimal("1.50"),
         "gross": Decimal("8.50"),
         "breakdown": {
-            "vat": {Decimal("20"): {"vat": Decimal("1.50"), "net": Decimal("6.00")}}
+            "vat-rate": {Decimal("0.20")},
+            "vat": {Decimal("20"): {"vat": Decimal("1.50"), "net": Decimal("6.00")}},
         },
     }
     assert headers == expected_headers
