@@ -1,11 +1,12 @@
 from decimal import Decimal
 
 from chellow.e.computer import SupplySource
-from chellow.e.tlms import hh
+from chellow.e.tlms import elexon_import, hh
 from chellow.models import (
     Comm,
     Contract,
     Cop,
+    DtcMeterType,
     EnergisationStatus,
     GspGroup,
     MarketRole,
@@ -21,6 +22,7 @@ from chellow.models import (
     VoltageLevel,
     insert_comms,
     insert_cops,
+    insert_dtc_meter_types,
     insert_energisation_statuses,
     insert_sources,
     insert_voltage_levels,
@@ -131,10 +133,12 @@ def test_hh(sess, mocker):
     )
     MtcLlfc.insert(sess, mtc_participant, llfc, vf, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     supply = site.insert_e_supply(
         sess,
         source,
@@ -155,7 +159,7 @@ def test_hh(sess, mocker):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 7867 6232 781",
         "510",
         imp_supplier_contract,
@@ -182,3 +186,38 @@ def test_hh(sess, mocker):
     hh_datum["gsp-kwh"] = 1
     hh(ss)
     assert hh_datum["tlm"] == 1.5
+
+
+def test_elexon_import(sess, mocker):
+    vf = to_utc(ct_datetime(1996, 1, 1))
+    participant = Participant.insert(sess, "CALB", "Calb")
+    market_role = MarketRole.insert(sess, "Z", "Non-core")
+    participant.insert_party(sess, market_role, "None core", vf, None, None)
+    Contract.insert_non_core(
+        sess, "tlms", "", {"enabled": True, "url": "https://example.com"}, vf, None, {}
+    )
+    Contract.insert_non_core(
+        sess,
+        "configuration",
+        "",
+        {"elexonportal_scripting_key": "xxk"},
+        vf,
+        None,
+        {},
+    )
+    sess.commit()
+
+    def log(msg):
+        pass
+
+    def set_progress(msg):
+        pass
+
+    s = mocker.Mock()
+    mock_response = mocker.Mock()
+    lines = [b"", b""]
+    mock_response.iter_lines = mocker.Mock(return_value=lines)
+    s.get = mocker.Mock(return_value=mock_response)
+    scripting_key = mocker.Mock()
+
+    elexon_import(sess, log, set_progress, s, scripting_key)
