@@ -4,6 +4,7 @@ from chellow.models import (
     Comm,
     Contract,
     Cop,
+    DtcMeterType,
     EnergisationStatus,
     GeneratorType,
     GspGroup,
@@ -22,6 +23,7 @@ from chellow.models import (
     VoltageLevel,
     insert_comms,
     insert_cops,
+    insert_dtc_meter_types,
     insert_energisation_statuses,
     insert_generator_types,
     insert_sources,
@@ -175,10 +177,12 @@ def displaced_virtual_bill(ds):
     insert_sources(sess)
     insert_generator_types(sess)
     generator_type = GeneratorType.get_by_code(sess, "turb")
-    source = Source.get_by_code(sess, "gen-net")
+    source = Source.get_by_code(sess, "gen-grid")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     site.insert_e_supply(
         sess,
         source,
@@ -199,7 +203,7 @@ def displaced_virtual_bill(ds):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 7867 6232 781",
         "510",
         imp_supplier_contract,
@@ -219,11 +223,7 @@ def displaced_virtual_bill(ds):
 
     mock_file = StringIO()
     mock_file.close = mocker.Mock()
-    mocker.patch("chellow.reports.report_109.open", return_value=mock_file)
-    mocker.patch(
-        "chellow.reports.report_109.chellow.dloads.make_names", return_value=("a", "b")
-    )
-    mocker.patch("chellow.reports.report_109.os.rename")
+    mocker.patch("chellow.reports.report_109.open_file", return_value=mock_file)
 
     end_year = 2000
     end_month = 1
