@@ -8,6 +8,7 @@ from chellow.models import (
     Comm,
     Contract,
     Cop,
+    DtcMeterType,
     EnergisationStatus,
     GspGroup,
     MarketRole,
@@ -26,6 +27,7 @@ from chellow.models import (
     insert_bill_types,
     insert_comms,
     insert_cops,
+    insert_dtc_meter_types,
     insert_energisation_statuses,
     insert_sources,
     insert_voltage_levels,
@@ -178,10 +180,12 @@ def virtual_bill(ds):
     )
     MtcLlfc.insert(sess, mtc_participant, llfc, valid_from, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     supply = site.insert_e_supply(
         sess,
         source,
@@ -202,7 +206,7 @@ def virtual_bill(ds):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 7867 6232 781",
         "510",
         imp_supplier_contract,
@@ -214,6 +218,7 @@ def virtual_bill(ds):
         None,
         None,
     )
+    supply_id = supply.id
     insert_bill_types(sess)
     bill_type = sess.execute(select(BillType).where(BillType.code == "N")).scalar_one()
     batch.insert_bill(
@@ -239,16 +244,11 @@ def virtual_bill(ds):
 
     mock_file = StringIO()
     mock_file.close = mocker.Mock()
-    mocker.patch("chellow.reports.report_387.open", return_value=mock_file)
-    mocker.patch(
-        "chellow.reports.report_387.chellow.dloads.make_names", return_value=("a", "b")
-    )
-    mocker.patch("chellow.reports.report_387.os.rename")
+    mocker.patch("chellow.reports.report_387.open_file", return_value=mock_file)
 
-    content(supply.id, start_date, start_date, user)
+    content(supply_id, start_date, start_date, user)
 
     actual = mock_file.getvalue()
-    print(actual)
 
     expected = [
         [
