@@ -8,6 +8,7 @@ from chellow.models import (
     Comm,
     Contract,
     Cop,
+    DtcMeterType,
     EnergisationStatus,
     GContract,
     GDn,
@@ -27,6 +28,7 @@ from chellow.models import (
     Pc,
     Report,
     ReportRun,
+    Scenario,
     Site,
     Snag,
     Source,
@@ -34,6 +36,7 @@ from chellow.models import (
     VoltageLevel,
     insert_comms,
     insert_cops,
+    insert_dtc_meter_types,
     insert_energisation_statuses,
     insert_g_reading_frequencies,
     insert_g_units,
@@ -69,6 +72,19 @@ def test_home(client, sess, mocker):
     match(response, 200, f"/e/dc_contracts/{dc_contract.id}/auto_importer")
 
 
+def test_input_date(client, sess, mocker):
+    query_string = {
+        "prefix": "timestamp",
+        "resolution": "day",
+        "timestamp_year": "2022",
+        "timestamp_month": "11",
+        "timestamp_day": "31",
+    }
+    response = client.get("/input_date", query_string=query_string)
+
+    match(response, 200)
+
+
 def test_local_report_home(client, sess):
     script = """from flask import render_template_string
 
@@ -122,6 +138,44 @@ def test_local_reports_post(client, sess):
     assert report.template == ""
 
 
+def test_scenario_get(sess, client):
+    props = {
+        "scenario_start_year": 2010,
+        "scenario_start_month": 5,
+    }
+    scenario = Scenario.insert(sess, "scenario 1", props)
+    sess.commit()
+
+    response = client.get(f"/scenarios/{scenario.id}")
+
+    match(response, 200)
+
+
+def test_scenario_edit_post(sess, client):
+    props = {
+        "scenario_start_year": 2010,
+        "scenario_start_month": 5,
+        "scenario_duration": 2,
+    }
+    scenario = Scenario.insert(sess, "scenario 1", props)
+    sess.commit()
+
+    data = {
+        "name": "scenario_bau",
+        "properties": """
+{
+  "local_rates": [],
+  "scenario_start_year": 2015,
+  "scenario_start_month": 6,
+  "scenario_duration": 1
+}""",
+    }
+
+    response = client.post(f"/scenarios/{scenario.id}/edit", data=data)
+
+    match(response, 303, r"/scenarios/1")
+
+
 def test_site_get(client, sess):
     valid_from = to_utc(ct_datetime(2000, 1, 1))
     site = Site.insert(sess, "CI017", "Water Works")
@@ -215,10 +269,12 @@ def test_site_get(client, sess):
     )
     MtcLlfc.insert(sess, mtc_participant, llfc, valid_from, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     insert_comms(sess)
     comm = Comm.get_by_code(sess, "GSM")
     site.insert_e_supply(
@@ -241,7 +297,7 @@ def test_site_get(client, sess):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 0470 7514 535",
         "510",
         imp_supplier_contract,
@@ -354,7 +410,7 @@ def test_site_get_dumb(client, sess):
         None,
     )
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
@@ -364,6 +420,8 @@ def test_site_get_dumb(client, sess):
     mtc_ssc = MtcSsc.insert(sess, mtc_participant, ssc, valid_from, None)
     mtc_llfc_ssc = MtcLlfcSsc.insert(sess, mtc_ssc, llfc, valid_from, None)
     MtcLlfcSscPc.insert(sess, mtc_llfc_ssc, pc, valid_from, None)
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     site.insert_e_supply(
         sess,
         source,
@@ -384,7 +442,7 @@ def test_site_get_dumb(client, sess):
         comm,
         ssc.code,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 0470 7514 535",
         "510",
         imp_supplier_contract,
@@ -404,19 +462,6 @@ def test_site_get_dumb(client, sess):
     match(response, 200, *patterns)
 
 
-def test_site_hh_data(sess, client):
-    site = Site.insert(sess, "CI017", "Water Works")
-    sess.commit()
-
-    query_string = {
-        "year": "2022",
-        "month": "3",
-    }
-    response = client.get(f"/sites/{site.id}/hh_data", query_string=query_string)
-
-    match(response, 200)
-
-
 def test_supplies_get_e(sess, client):
     valid_from = to_utc(ct_datetime(1996, 1, 1))
     site = Site.insert(sess, "CI017", "Water Works")
@@ -508,7 +553,7 @@ def test_supplies_get_e(sess, client):
         None,
     )
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
@@ -518,6 +563,8 @@ def test_supplies_get_e(sess, client):
     mtc_ssc = MtcSsc.insert(sess, mtc_participant, ssc, valid_from, None)
     mtc_llfc_ssc = MtcLlfcSsc.insert(sess, mtc_ssc, llfc, valid_from, None)
     MtcLlfcSscPc.insert(sess, mtc_llfc_ssc, pc, valid_from, None)
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     site.insert_e_supply(
         sess,
         source,
@@ -538,7 +585,7 @@ def test_supplies_get_e(sess, client):
         comm,
         ssc.code,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 0470 7514 535",
         "510",
         imp_supplier_contract,
@@ -570,7 +617,7 @@ def test_supplies_get_e(sess, client):
         comm,
         ssc.code,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 0471 7514 532",
         "510",
         imp_supplier_contract,
@@ -625,6 +672,8 @@ def test_supplies_get_g(sess, client):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
     sess.commit()
     query_string = {
@@ -739,10 +788,12 @@ def test_general_import_post_full(sess, client):
     )
     MtcLlfc.insert(sess, mtc_participant, llfc, valid_from, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     supply = site.insert_e_supply(
         sess,
         source,
@@ -763,7 +814,7 @@ def test_general_import_post_full(sess, client):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 0470 7514 535",
         "510",
         imp_supplier_contract,
@@ -903,6 +954,23 @@ def test_non_core_rate_script_edit_get(sess, client):
     match(response, 200)
 
 
+def test_non_core_rate_script_edit_get_bst(sess, client):
+    market_role_Z = MarketRole.get_by_code(sess, "Z")
+    participant = Participant.insert(sess, "CALB", "AK Industries")
+    participant.insert_party(
+        sess, market_role_Z, "None core", utc_datetime(2000, 1, 1), None, None
+    )
+    contract = Contract.insert_non_core(
+        sess, "rcrc", "", {}, to_utc(ct_datetime(2000, 5, 31, 23, 30)), None, {}
+    )
+    rs = contract.start_rate_script
+    sess.commit()
+
+    response = client.get(f"/non_core_rate_scripts/{rs.id}/edit")
+
+    match(response, 200, r'<input type="hidden" name="start_hour" value="23">')
+
+
 def test_site_edit_get(sess, client):
     vf = to_utc(ct_datetime(2000, 1, 1))
     site = Site.insert(sess, "CI017", "Water Works")
@@ -922,6 +990,14 @@ def test_rate_server_get(sess, client):
     match(response, 200)
 
 
+def test_report_run_get_bill_check(sess, client):
+    report_run = ReportRun.insert(sess, "bill_check", None, "_b_88", {})
+    sess.commit()
+    query_string = {"element": "net"}
+    response = client.get(f"/report_runs/{report_run.id}", query_string=query_string)
+    match(response, 200)
+
+
 def test_report_run_spreadsheet_get(sess, client):
     report_run = ReportRun.insert(sess, "bill_check", None, "_b_88", {})
     report_run.insert_row(sess, "", ["clump"], {}, {})
@@ -929,3 +1005,24 @@ def test_report_run_spreadsheet_get(sess, client):
 
     response = client.get(f"/report_runs/{report_run.id}/spreadsheet")
     match(response, 200)
+
+
+def test_report_run_row_get_bill_check(sess, client):
+    report_run = ReportRun.insert(sess, "bill_check", None, "_b_88", {})
+    report_run_row = report_run.insert_row(
+        sess,
+        "",
+        ["clump"],
+        {},
+        {},
+        data={
+            "actual_net_gbp": 10,
+            "virtual_net_gbp": 20.32,
+            "difference_net_gbp": 23.6,
+            "elements": {},
+        },
+    )
+    sess.commit()
+
+    response = client.get(f"/report_run_rows/{report_run_row.id}")
+    match(response, 200)
